import ctypes 
import time
import struct
import sys
import types
import keyboard
from pyiec101.iec101api import *


# enbale to view traffic
VIEW_TRAFFIC = 1

# print error code and description
def errorcodestring(errorcode):
    sIEC101ErrorCodeDes = sIEC101ErrorCode()
    sIEC101ErrorCodeDes.iErrorCode = errorcode
    iec101_lib.IEC101ErrorCodeString(sIEC101ErrorCodeDes)
    return sIEC101ErrorCodeDes.LongDes.decode("utf-8")

# print error value and description
def errorvaluestring(errorvalue):
    sIEC101ErrorValueDes = sIEC101ErrorValue()
    sIEC101ErrorValueDes.iErrorValue = errorvalue   
    iec101_lib.IEC101ErrorValueString(sIEC101ErrorValueDes)
    return sIEC101ErrorValueDes.LongDes.decode("utf-8")

# print the struct sIEC101DataAttributeID and sIEC101DataAttributeData
def vPrintDataInformation(psIEC101DataAttributeID , psIEC101DataAttributeData ):

    print(f" Data Link Address {psIEC101DataAttributeID.contents.u16DataLinkAddress}")
    print(f" Common Address {psIEC101DataAttributeID.contents.u16CommonAddress}")
    print(f" Typeid ID is {psIEC101DataAttributeID.contents.eTypeID} IOA   {psIEC101DataAttributeID.contents.u32IOA}")
    print(f" Datatype->{psIEC101DataAttributeData.contents.eDataType} Datasize->{ psIEC101DataAttributeData.contents.eDataSize}" )

    if(psIEC101DataAttributeData.contents.tQuality) != eIEC870QualityFlags.GD :
        if(psIEC101DataAttributeData.contents.tQuality & eIEC870QualityFlags.IV) == eIEC870QualityFlags.IV:
            print(" IEC_INVALID_FLAG")
        if(psIEC101DataAttributeData.contents.tQuality & eIEC870QualityFlags.NT) == eIEC870QualityFlags.NT:
             print(" IEC_NONTOPICAL_FLAG")
        if(psIEC101DataAttributeData.contents.tQuality & eIEC870QualityFlags.SB) == eIEC870QualityFlags.SB:
             print(" IEC_SUBSTITUTED_FLAG")
        if(psIEC101DataAttributeData.contents.tQuality & eIEC870QualityFlags.BL) == eIEC870QualityFlags.BL:
             print(" IEC_BLOCKED_FLAG")

    data_type = psIEC101DataAttributeData.contents.eDataType

    if data_type in (eDataTypes.SINGLE_POINT_DATA, eDataTypes.DOUBLE_POINT_DATA, eDataTypes.UNSIGNED_BYTE_DATA):
        data = bytearray(ctypes.string_at(psIEC101DataAttributeData.contents.pvData, 1))
        u8data = struct.unpack('B', data)[0] 
        print(f" Data : {u8data}")

    elif data_type == eDataTypes.SIGNED_BYTE_DATA:
        data = bytearray(ctypes.string_at(psIEC101DataAttributeData.contents.pvData, 1))
        i8data = struct.unpack('b', data)[0]        
        print(f" Data : {i8data}")

    elif data_type == eDataTypes.UNSIGNED_WORD_DATA:
        data = bytearray(ctypes.string_at(psIEC101DataAttributeData.contents.pvData, 2))
        u16data = struct.unpack('H', data)[0]        
        print(f" Data : {u16data}")

    elif data_type == eDataTypes.SIGNED_WORD_DATA:
        data = bytearray(ctypes.string_at(psIEC101DataAttributeData.contents.pvData, 2))
        i16data = struct.unpack('h', data)[0]        
        print(f" Data : {i16data}")

    elif data_type == eDataTypes.UNSIGNED_DWORD_DATA:
        data = bytearray(ctypes.string_at(psIEC101DataAttributeData.contents.pvData, 4))
        u32data = struct.unpack('I', data)[0]        
        print(f" Data : {u32data}")


    elif data_type == eDataTypes.SIGNED_DWORD_DATA:
        data = bytearray(ctypes.string_at(psIEC101DataAttributeData.contents.pvData, 4))
        i32data = struct.unpack('i', data)[0]        
        print(f" Data : {i32data}")

    elif data_type == eDataTypes.FLOAT32_DATA:
        data = bytearray(ctypes.string_at(psIEC101DataAttributeData.contents.pvData, 4))
        f32data = struct.unpack('f', data)[0] 
        print(f" Data : {f32data:.3f}")

    if psIEC101DataAttributeData.contents.sTimeStamp.u16Year != 0:
        print(f" Date : {psIEC101DataAttributeData.contents.sTimeStamp.u8Day:02}-{psIEC101DataAttributeData.contents.sTimeStamp.u8Month:02}-{psIEC101DataAttributeData.contents.sTimeStamp.u16Year:04}  DOW -{psIEC101DataAttributeData.contents.sTimeStamp.u8DayoftheWeek}")
        print(f" Time : {psIEC101DataAttributeData.contents.sTimeStamp.u8Hour:02}:{psIEC101DataAttributeData.contents.sTimeStamp.u8Minute:02}:{psIEC101DataAttributeData.contents.sTimeStamp.u8Seconds:02}:{psIEC101DataAttributeData.contents.sTimeStamp.u16MilliSeconds:03}")

# iec101 read callback 
def cbRead(u16ObjectId, psIEC104DataAttributeID, psIEC104DataAttributeData , psReadParams, ptErrorValue):
    i16rErrorCode = ctypes.c_short()
    i16rErrorCode = 0   
    print(" cbRead() called ")
    print(" Server ID : %u" % u16ObjectId)
    vPrintDataInformation(psIEC104DataAttributeID, psIEC104DataAttributeData)
    message = f"Orginator Address {psReadParams.contents.u8OriginatorAddress}"
    print(message)
    return i16rErrorCode

# iec101 write callback 
def cbWrite(u16ObjectId, psIEC101DataAttributeID, psIEC101DataAttributeData, psIEC101WriteParameters, ptErrorValue):
    i16rErrorCode = ctypes.c_short()
    i16rErrorCode = 0   
    print(" cbWrite() called ")
    print(" Server ID : %u" % u16ObjectId)
    vPrintDataInformation(psIEC101DataAttributeID, psIEC101DataAttributeData)
    message = f"Orginator Address {psIEC101WriteParameters.contents.u8OriginatorAddress}"
                                                                    
    print(message)
    return i16rErrorCode

# Freeze Callback
def cbFreeze( u16ObjectId, eCounterFreeze, psIEC101DataAttributeID , psIEC101DataAttributeData , psIEC101WriteParameters , ptErrorValue):

    i16rErrorCode = ctypes.c_short()
    i16rErrorCode = 0   
    print(" cbFreeze() called")  

    print(" Server ID : %u" % u16ObjectId)
    vPrintDataInformation(psIEC101DataAttributeID, psIEC101DataAttributeData)
    message = f"Orginator Address {psIEC101WriteParameters.contents.u8OriginatorAddress}"
    print(message)

    print(f" Command Typeid : {psIEC101DataAttributeID.contents.eTypeID}");
    print(f" COT : {psIEC101WriteParameters.contents.eCause}");
   

    return i16rErrorCode

# Select callback
def cbSelect(u16ObjectId, psIEC101DataAttributeID , psIEC101DataAttributeData ,psIEC101CommandParameters , ptErrorValue):

    i16rErrorCode = ctypes.c_short()
    i16rErrorCode = 0   
    print(" cbSelect() called ")

    print(" Server ID : %u" % u16ObjectId)
    vPrintDataInformation(psIEC101DataAttributeID, psIEC101DataAttributeData)
    message = f"Orginator Address {psIEC101CommandParameters.contents.u8OriginatorAddress}"
    print(message)
    
    print(f" Qualifier : {psIEC101CommandParameters.contents.eQOCQU}" );
    print(f" Pulse Duration : {psIEC101CommandParameters.contents.u32PulseDuration}" );

    return i16rErrorCode

# Operate callback
def cbOperate(u16ObjectId, psIEC101DataAttributeID , psIEC101DataAttributeData ,psIEC101CommandParameters , ptErrorValue):

    i16rErrorCode = ctypes.c_short()
    i16rErrorCode = 0   
    print(" cbOperate() called")

    print(" Server ID : %u" % u16ObjectId)
    vPrintDataInformation(psIEC101DataAttributeID, psIEC101DataAttributeData)
    message = f"Orginator Address {psIEC101CommandParameters.contents.u8OriginatorAddress}"
    print(message)

    print(f" Qualifier : {psIEC101CommandParameters.contents.eQOCQU}")
    print(f" Pulse Duration : {psIEC101CommandParameters.contents.u32PulseDuration}")
   

    return i16rErrorCode

# Operate pulse end callback
def cbpulseend(u16ObjectId, psIEC101DataAttributeID , psIEC101DataAttributeData ,psIEC101CommandParameters , ptErrorValue):

    i16rErrorCode = ctypes.c_short()
    i16rErrorCode = 0   
    print(" cbpulseend() called ")


    print(" Server ID : %u" % u16ObjectId)
    vPrintDataInformation(psIEC101DataAttributeID, psIEC101DataAttributeData)
    message = f"Orginator Address {psIEC101CommandParameters.contents.u8OriginatorAddress}"
    print(message)

    print(f" Qualifier : {psIEC101CommandParameters.contents.eQOCQU}")
    print(f" Pulse Duration : {psIEC101CommandParameters.contents.u32PulseDuration}")
   

    return i16rErrorCode

# Cancel callback
def cbCancel(u16ObjectId, eOperation, psIEC101DataAttributeID , psIEC101DataAttributeData ,psIEC101CommandParameters , ptErrorValue):

    i16rErrorCode = ctypes.c_short()
    i16rErrorCode = 0   
    print(" cbCancel() called from IEC101 client")

    print(" Server ID : %u" % u16ObjectId)
    vPrintDataInformation(psIEC101DataAttributeID, psIEC101DataAttributeData)
    message = f"Orginator Address {psIEC101CommandParameters.contents.u8OriginatorAddress}"
    print(message)


    if eOperation   ==  eOperationFlag.OPERATE:
        print(" Operate operation to be cancel")
    elif eOperation   ==  eOperationFlag.SELECT:
        print(" Select operation to cancel")

    print(f" Qualifier : {psIEC101CommandParameters.contents.eQOCQU}")
    print(f" Pulse Duration :{psIEC101CommandParameters.contents.u32PulseDuration}")
    

    return i16rErrorCode

# Parameteract callback
def cbParameterAct(u16ObjectId, psIEC101DataAttributeID , psIEC101DataAttributeData , psIEC101ParameterActParameters , ptErrorValue):

    i16rErrorCode = ctypes.c_short()
    i16rErrorCode = 0   
    print(" cbParameterAct called from IEC101 client")

    print(" Server ID : %u" % u16ObjectId)
    vPrintDataInformation(psIEC101DataAttributeID, psIEC101DataAttributeData)
    message = f"Orginator Address {psIEC101ParameterActParameters.contents.u8OriginatorAddress}"
    print(message)

    print(f" Qualifier of Parameter Activation/Kind of Parameter - {psIEC101ParameterActParameters.contents.u8QPA}")
    

    return i16rErrorCode

# Debug callback
def cbDebug(u16ObjectId,  psIEC101DebugData , ptErrorValue):
    i16rErrorCode = ctypes.c_short()
    i16rErrorCode = 0 

    u16nav = ctypes.c_ushort()
    u16nav = 0
   
    #printf(" cbDebug() called");
    
    print(f" {psIEC101DebugData.contents.sTimeStamp.u8Hour}:{psIEC101DebugData.contents.sTimeStamp.u8Minute}:{psIEC101DebugData.contents.sTimeStamp.u8Seconds} Server ID: {u16ObjectId}", end='')

    if (psIEC101DebugData.contents.u32DebugOptions & eDebugOptionsFlag.DEBUG_OPTION_TX) == eDebugOptionsFlag.DEBUG_OPTION_TX:
        print(f" Serial Port {psIEC101DebugData.contents.u16ComportNumber} ", end='')
        print(" ->", end='')      
       
        for u16nav in range(psIEC101DebugData.contents.u16TxCount):
            print(f" {psIEC101DebugData.contents.au8TxData[u16nav]:02x}", end='')
        

        
    if (psIEC101DebugData.contents.u32DebugOptions & eDebugOptionsFlag.DEBUG_OPTION_RX) == eDebugOptionsFlag.DEBUG_OPTION_RX:
         print(f" Serial Port {psIEC101DebugData.contents.u16ComportNumber} ", end='')
         print(" <-", end='')

         for u16nav in range(psIEC101DebugData.contents.u16RxCount):
            print(f" {psIEC101DebugData.contents.au8RxData[u16nav]:02x}", end='')
        

        
    if (psIEC101DebugData.contents.u32DebugOptions & eDebugOptionsFlag.DEBUG_OPTION_ERROR) == eDebugOptionsFlag.DEBUG_OPTION_ERROR:
        print(f" Error message {psIEC101DebugData.contents.au8ErrorMessage}")
        print(f" ErrorCode {psIEC101DebugData.contents.iErrorCode}")
        print(f" ErrorValue {psIEC101DebugData.contents.tErrorvalue}")

    print("", flush=True)

    return i16rErrorCode

# update particular typeid and IOA FROM USER INPUT
def update(myServer):

    i16ErrorCode = ctypes.c_short()
    tErrorValue =  ctypes.c_short()
    
    print("UPDATE CALLED")
    while True:
        try:
            u32ioa = ctypes.c_uint32(int(input("MeasuredFloat(M_ME_TF_1) Enter Information object address(IOA)- 100 to 109 ")))
        except ValueError:
            print("Please enter a number 100 to 109")
        else:
            break

    while True:
        try:
            f32value = ctypes.c_float(float(input("Enter update float value: ")))
        except ValueError:
            print("Please enter a float number ")
        else:
            break

   
    psDAID = sIEC101DataAttributeID()
    psNewValue  = sIEC101DataAttributeData()

    psDAID.u16SerialPortNumber    =   1
    psDAID.u16DataLinkAddress    =   1
    psDAID.u16CommonAddress                     =  1
    psDAID.eTypeID                              =  eIEC870TypeID.M_ME_TF_1
    psDAID.u32IOA                               =   u32ioa
    psDAID.pvUserData                           =   None
    psNewValue.tQuality                         =   eIEC870QualityFlags.GD

    psNewValue.pvData                           =   ctypes.cast(ctypes.pointer(f32value),ctypes.c_void_p)
    psNewValue.eDataType                        =   eDataTypes.FLOAT32_DATA
    psNewValue.eDataSize                        =   eDataSizes.FLOAT32_SIZE

    now = time.time()
    timeinfo = time.localtime(now)
    
    #current date
    psNewValue.sTimeStamp.u8Day = timeinfo.tm_mday
    psNewValue.sTimeStamp.u8Month = timeinfo.tm_mon
    psNewValue.sTimeStamp.u16Year = timeinfo.tm_year 

    psNewValue.sTimeStamp.u8Hour = timeinfo.tm_hour
    psNewValue.sTimeStamp.u8Minute = timeinfo.tm_min
    psNewValue.sTimeStamp.u8Seconds = timeinfo.tm_sec
    psNewValue.sTimeStamp.u16MilliSeconds = 0
    psNewValue.sTimeStamp.u16MicroSeconds = 0
    psNewValue.sTimeStamp.i8DSTTime = 0
    psNewValue.sTimeStamp.u8DayoftheWeek = 4
    psNewValue.bTimeInvalid = False

    #printf(" update float value %f",f32data);
    # Update server
   
    i16ErrorCode = iec101_lib.IEC101Update(myServer, True,ctypes.byref(psDAID),ctypes.byref(psNewValue),1,ctypes.byref((tErrorValue)))
    if i16ErrorCode != 0:
        message = f"IEC 60870-5-101 Library API Function - IEC101Update() failed: {i16ErrorCode} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
        print(message)     

    else:
        message = f"IEC 60870-5-101 Library API Function - IEC101Update() success: {i16ErrorCode} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
        print(message) 

# main program
def main():
    print(" \t\t**** IEC 60870-5-101 Protocol Server Library Test ****")
    # Check library version against the library header file
    if iec101_lib.IEC101GetLibraryVersion().decode("utf-8") != IEC101_VERSION:
        print(" Error: Version Number Mismatch")
        print(" Library Version is  : {}".format(iec101_lib.IEC101GetLibraryVersion().decode("utf-8")))
        print(" The Header Version used is : {}".format(IEC101_VERSION))
        print("")
        input(" Press Enter to free IEC 101 Server object")
        exit(0)

    print(" Library Version is : {}".format(iec101_lib.IEC101GetLibraryVersion().decode("utf-8")))
    print(" Library Build on   : {}".format(iec101_lib.IEC101GetLibraryBuildTime().decode("utf-8")))
    print(" Library License Information   : {}".format(iec101_lib.IEC101GetLibraryLicenseInfo().decode("utf-8")))

    i16ErrorCode = ctypes.c_short()
    tErrorValue =  ctypes.c_short()

    sParameters = sIEC101Parameters()

   

    # Initialize IEC 60870-5-104 Server object parameters
    sParameters.eAppFlag          =  eApplicationFlag.APP_SERVER        # This is a IEC104 Server      
    sParameters.ptReadCallback    = IEC101ReadCallback(cbRead)               # Read Callback
    sParameters.ptWriteCallback   = IEC101WriteCallback(cbWrite)                # Write Callback
    sParameters.ptUpdateCallback  = ctypes.cast(None,IEC101UpdateCallback) #IEC104UpdateCallback(0)                 # Update Callback
    sParameters.ptSelectCallback  = IEC101ControlSelectCallback(cbSelect)               # Select Callback
    sParameters.ptOperateCallback = IEC101ControlOperateCallback(cbOperate)              # Operate Callback
    sParameters.ptCancelCallback  = IEC101ControlCancelCallback(cbCancel)              # Cancel Callback
    sParameters.ptFreezeCallback  = IEC101ControlFreezeCallback(cbFreeze)              # Freeze Callback
    sParameters.ptDebugCallback   = IEC101DebugMessageCallback(cbDebug)                # Debug Callback
    sParameters.ptPulseEndActTermCallback = IEC101ControlPulseEndActTermCallback(cbpulseend)    # pulse end callback
    sParameters.ptParameterActCallback = IEC101ParameterActCallback(cbParameterAct)   # Parameter activation callback
    sParameters.ptDirectoryCallback    = IEC101DirectoryCallback(0)              # Directory Callback
    sParameters.ptClientStatusCallback   = IEC101ClientStatusCallback(0)           # client connection status Callback
    sParameters.u32Options        = 0
    sParameters.u16ObjectId				= 1				#Server ID which used in callbacks to identify the iec 104 server object   


    # Create a server object

    myServer =  iec101_lib.IEC101Create(ctypes.byref(sParameters), ctypes.byref((i16ErrorCode)), ctypes.byref((tErrorValue)))
    if i16ErrorCode.value != 0:
        message = f"IEC 60870-5-101 Library API Function - IEC101Create() failed: {i16ErrorCode.value} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
        print(message)    
        exit(0) 
    else:
        message = f"IEC 60870-5-101 Library API Function - IEC101Create() success: {i16ErrorCode.value} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
        print(message) 


    while(True):

        sIEC101Config = sIEC101ConfigurationParameters()       


        sIEC101Config.sServerSet.u8NumberofSerialPortConnections    =   1
        # Allocate memory for objects
        
        

        arraypointer = (sSerialCommunicationSettings  * sIEC101Config.sServerSet.u8NumberofSerialPortConnections )()
        sIEC101Config.sServerSet.psSerialSet = ctypes.cast(arraypointer, ctypes.POINTER(sSerialCommunicationSettings))

        arraypointer[0].eSerialType   =  eSerialTypes.SERIAL_RS232
        # check computer configuration serial com port number,  if server and client application running in same system, we can use com0com
        arraypointer[0].u16SerialPortNumber   =   1
        arraypointer[0].eSerialBitRate       =   eSerialBitRate.BITRATE_9600
        arraypointer[0].eWordLength          =   eSerialWordLength.WORDLEN_8BITS
        arraypointer[0].eSerialParity        =   eSerialParity.EVEN
        arraypointer[0].eStopBits            =   eSerialStopBits.STOPBIT_1BIT

        #serial port flow control
        arraypointer[0].sFlowControl.bWinCTSoutputflow         =  False
        arraypointer[0].sFlowControl.bWinDSRoutputflow         =  False
        arraypointer[0].sFlowControl.eWinDTR					  =	 eWinDTRcontrol.WIN_DTR_CONTROL_DISABLE
        arraypointer[0].sFlowControl.eWinRTS					  =   eWinRTScontrol.WIN_RTS_CONTROL_DISABLE
        arraypointer[0].sFlowControl.eLinuxFlowControl         = eLinuxSerialFlowControl.FLOW_NONE

        arraypointer[0].sRxTimeParam.u16CharacterTimeout     =   1
        arraypointer[0].sRxTimeParam.u16MessageTimeout       =   0
        arraypointer[0].sRxTimeParam.u16InterCharacterDelay  =   5
        arraypointer[0].sRxTimeParam.u16PostDelay            =   0
        arraypointer[0].sRxTimeParam.u16PreDelay             =   0
        arraypointer[0].sRxTimeParam.u8CharacterRetries      =   20
        arraypointer[0].sRxTimeParam.u8MessageRetries        =   0

        sIEC101Config.sServerSet.sServerProtSet.eDataLink                =   eDataLinkTransmission.UNBALANCED_MODE

        sIEC101Config.sServerSet.sServerProtSet.elinkAddrSize            =   eDataLinkAddressSize.DL_TWO_BYTE
        sIEC101Config.sServerSet.sServerProtSet.u16DataLinkAddress       =   1
        sIEC101Config.sServerSet.sServerProtSet.eCOTsize                 =   eCauseofTransmissionSize.COT_TWO_BYTE
        sIEC101Config.sServerSet.sServerProtSet.eIOAsize                 =   eInformationObjectAddressSize.IOA_THREE_BYTE
        sIEC101Config.sServerSet.sServerProtSet.eCASize                  =   eCommonAddressSize.CA_TWO_BYTE
        sIEC101Config.sServerSet.sServerProtSet.u8TotalNumberofStations  =   1
        sIEC101Config.sServerSet.sServerProtSet.au16CommonAddress[0]     =   1
        sIEC101Config.sServerSet.sServerProtSet.au16CommonAddress[1]     =   0
        sIEC101Config.sServerSet.sServerProtSet.au16CommonAddress[2]     =   0
        sIEC101Config.sServerSet.sServerProtSet.au16CommonAddress[3]     =   0
        sIEC101Config.sServerSet.sServerProtSet.au16CommonAddress[4]     =   0



        sIEC101Config.sServerSet.sServerProtSet.eNegACK  = eNegativeACK.FIXED_FRAME_NACK
        sIEC101Config.sServerSet.sServerProtSet.ePosACK  =   ePositiveACK.SINGLE_CHAR_ACK_E5

        sIEC101Config.sServerSet.sServerProtSet.u16Class1EventBufferSize =   5000
        sIEC101Config.sServerSet.sServerProtSet.u16Class2EventBufferSize =   5000

        sIEC101Config.sServerSet.sServerProtSet.u8Class1BufferOverFlowPercentage     =   90
        sIEC101Config.sServerSet.sServerProtSet.u8Class2BufferOverFlowPercentage     =   90
        sIEC101Config.sServerSet.sServerProtSet.u8MaxAPDUSize                        =   253
        sIEC101Config.sServerSet.sServerProtSet.u16ShortPulseTime                    =   5000
        sIEC101Config.sServerSet.sServerProtSet.u16LongPulseTime                     =   10000
        sIEC101Config.sServerSet.sServerProtSet.u32ClockSyncPeriod                   =   0
        sIEC101Config.sServerSet.sServerProtSet.bGenerateACTTERMrespond              =   True


        sIEC101Config.sServerSet.sServerProtSet.u32BalancedModeTestConnectionSignalInterval  = 60

        # File transfer protocol configuration parameters
        sIEC101Config.sServerSet.sServerProtSet.bEnableFileTransfer = False
        sIEC101Config.sServerSet.sServerProtSet.ai8FileTransferDirPath =  "//FileTransferServer//".encode('utf-8')
        sIEC101Config.sServerSet.sServerProtSet.u16MaxFilesInDirectory    = 10



        sIEC101Config.sServerSet.sDebug.u32DebugOptions     =   (eDebugOptionsFlag.DEBUG_OPTION_RX | eDebugOptionsFlag.DEBUG_OPTION_TX)


        # Debug option settings
        if  'VIEW_TRAFFIC' in globals():
            sIEC101Config.sServerSet.sDebug.u32DebugOptions   =   (eDebugOptionsFlag.DEBUG_OPTION_RX | eDebugOptionsFlag.DEBUG_OPTION_TX)
        else:
            sIEC101Config.sServerSet.sDebug.u32DebugOptions  =   0

        sIEC101Config.sServerSet.sServerProtSet.bTransmitSpontMeasuredValue = True
        sIEC101Config.sServerSet.sServerProtSet.bTransmitInterrogationMeasuredValue = True
        sIEC101Config.sServerSet.sServerProtSet.bTransmitBackScanMeasuredValue = True

        sIEC101Config.sServerSet.sServerProtSet.u8InitialdatabaseQualityFlag = eIEC870QualityFlags.GD  # 0- good/valid, 1 BIT- iv, 2 BIT-nt,  MAX VALUE -3   */
        sIEC101Config.sServerSet.sServerProtSet.bUpdateCheckTimestamp = True # if it true ,the timestamp change also generate event  during the iec104update */

        # Allocate memory for objects
        sIEC101Config.sServerSet.u16NoofObject           = 2;       # Define number of objects





        sIEC101Config.sServerSet.psIEC101Objects  = ( sIEC101Object * sIEC101Config.sServerSet.u16NoofObject)()




        sIEC101Config.sServerSet.psIEC101Objects[0].ai8Name = "M_ME_TF_1 100-109".encode('utf-8')
        sIEC101Config.sServerSet.psIEC101Objects[0].eTypeID     =  eIEC870TypeID.M_ME_TF_1
        sIEC101Config.sServerSet.psIEC101Objects[0].u32IOA          = 100
        sIEC101Config.sServerSet.psIEC101Objects[0].u16Range        = 10
        sIEC101Config.sServerSet.psIEC101Objects[0].eIntroCOT       = eIEC870COTCause.INRO6
        sIEC101Config.sServerSet.psIEC101Objects[0].eControlModel   =   eControlModelConfig.STATUS_ONLY
        sIEC101Config.sServerSet.psIEC101Objects[0].u32SBOTimeOut   =   0
        sIEC101Config.sServerSet.psIEC101Objects[0].u16CommonAddress    =   1
        sIEC101Config.sServerSet.psIEC101Objects[0].eClass          =   eIECClass.IEC_CLASS1

        #Second object detail
        sIEC101Config.sServerSet.psIEC101Objects[1].ai8Name = "C_SE_TC_1".encode('utf-8')
        sIEC101Config.sServerSet.psIEC101Objects[1].eTypeID     =  eIEC870TypeID.C_SE_TC_1
        sIEC101Config.sServerSet.psIEC101Objects[1].u32IOA          = 100
        sIEC101Config.sServerSet.psIEC101Objects[1].eIntroCOT       = eIEC870COTCause.NOTUSED
        sIEC101Config.sServerSet.psIEC101Objects[1].u16Range        = 10
        sIEC101Config.sServerSet.psIEC101Objects[1].eControlModel  = eControlModelConfig.DIRECT_OPERATE
        sIEC101Config.sServerSet.psIEC101Objects[1].u32SBOTimeOut   = 0
        sIEC101Config.sServerSet.psIEC101Objects[1].u16CommonAddress    =   1
        sIEC101Config.sServerSet.psIEC101Objects[1].eClass          =   eIECClass.IEC_NO_CLASS

            

        i16ErrorCode =  iec101_lib.IEC101LoadConfiguration(myServer, ctypes.byref(sIEC101Config), ctypes.byref((tErrorValue)))
        if i16ErrorCode != 0:
            message = f"IEC 60870-5-101 Library API Function - IEC101LoadConfiguration() failed: {i16ErrorCode} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
            print(message)    
            break

        else:
            message = f"IEC 60870-5-101 Library API Function - IEC101LoadConfiguration() success: {i16ErrorCode} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
            print(message) 



        i16ErrorCode =  iec101_lib.IEC101Start(myServer, ctypes.byref((tErrorValue)))
        if i16ErrorCode != 0:
            message = f"IEC 60870-5-101 Library API Function - IEC101Start() failed: {i16ErrorCode} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
            print(message)    
            break

        else:
            message = f"IEC 60870-5-101 Library API Function - IEC101Start() success: {i16ErrorCode} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
            print(message) 

        print("press x to exit")

        while(True):
            if keyboard.is_pressed('x'):
                print("x pressed, exiting loop")
                keyboard.release('x')
                time.sleep(0.1)
                break
            elif keyboard.is_pressed('u'):
                print("u pressed, update called")
                keyboard.release('u')
                time.sleep(0.1)
                update(myServer)

            #Xprint("sleep called")
            time.sleep(50 /1000)

        break
            
            

      



    i16ErrorCode =  iec101_lib.IEC101Stop(myServer, ctypes.byref((tErrorValue)))
    if i16ErrorCode != 0:
        message = f"IEC 60870-5-101 Library API Function - IEC101Stop() failed: {i16ErrorCode} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
        print(message)        
    else:
        message = f"IEC 60870-5-101 Library API Function - IEC101Stop() success: {i16ErrorCode} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
        print(message) 



    i16ErrorCode =  iec101_lib.IEC101Free(myServer, ctypes.byref((tErrorValue)))
    if i16ErrorCode != 0:
        message = f"IEC 60870-5-101 Library API Function - IEC101Free() failed: {i16ErrorCode} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
        print(message)    
    else:
        message = f"IEC 60870-5-101 Library API Function - IEC101Free() success: {i16ErrorCode} - {errorcodestring(i16ErrorCode)}, {tErrorValue.value} - {errorvaluestring(tErrorValue)}"
        print(message) 




    print("Exiting the program...")
    

if __name__ == "__main__":
    main()


import ctypes
import enum

#
#   Prerequisits:
#   -------------
#   If you are using Python < 3.4 run `pip install enum34`.
#
#   Problem Definition
#   ------------------
#   ctypes.Structure class doesn't support enumerations out-of-the-box.
#   Consider the following (non-working) example:
#
#   class EnableDisableType(enum.IntEnum):
#       ENABLE = 1
#       DISABLE = 2

#   class SomeStructure(ctypes.Structure):
#       _fields_ = [
#           ("value", ctypes.c_uint32),
#           ("switch", EnableDisableType),
#       ]
#
#    Solution:
#    ---------
#    1). Use the following `StructureWithEnums` class instead of `ctypes.Structure`.
#    2). Add `_map`attribute to your structure definition (which maps field names to enumeration types).
#    3)  Replace enumeration types in your `_fields_` list with `ctypes.c_int`.
#    Voilà!
#
#    class SomeStructure(StructureWithEnums):
#        _fields_ = [
#            ("value", ctypes.c_uint32),
#            ("switch", ctypes.c_int),
#        ]
#        _map = {
#            "switch":  EnableDisableType
#        }
#

class StructureWithEnums(ctypes.Structure):
    """Add missing enum feature to ctypes Structures.
    """
    _map = {}

    def __getattribute__(self, name):
        _map = ctypes.Structure.__getattribute__(self, '_map')
        value = ctypes.Structure.__getattribute__(self, name)
        if name in _map:
            EnumClass = _map[name]
            if isinstance(value, ctypes.Array):
                return [EnumClass(x) for x in value]
            else:
                return EnumClass(value)
        else:
            return value

    def __str__(self):
        result = []
        result.append("struct {0} {{".format(self.__class__.__name__))
        for field in self._fields_:
            attr, attrType = field
            if attr in self._map:
                attrType = self._map[attr]
            value = getattr(self, attr)
            result.append("    {0} [{1}] = {2!r};".format(attr, attrType.__name__, value))
        result.append("};")
        return '\n'.join(result)

    __repr__ = __str__
##



import ctypes
from enum import *


	# Max Directory Path Size
MAX_DIRECTORY_PATH = 248
	# Max Value of station address/ common address present in the physical link
MAX_CA = 100
	
	
	
	
	
	
		 
	 # Flags for struct sIEC101Parameters.uiOptions flags 
class eApplicationOptionFlag(Enum):        
			APP_OPTION_NONE                                 = 0x00 # No options set 
			APP_SEQUENCE_BIT                                = 0x01 # Server builds iec frame with sequence for monitoring information without time stamp 

		
		
				# Flags for IEC101Cancel/cancel callback operation flags 
class eOperationFlag(Enum):        
			SELECT                  = 1 # Select Operation 
			OPERATE                 = 2 # Operate Operation
		

		# for IEC101Client status  flags 
class eStatus(IntEnum):        
			NOT_CONNECTED   =   0 # not connected 
			CONNECTED       =   1  # connected 
		
		
				#  Flags for eCauseofTransmissionSize - Cause of Transmission size
class eCauseofTransmissionSize(IntEnum):        
			COT_ONE_BYTE        = 1        # 1 Octet cause of transmission 
			COT_TWO_BYTE        = 2        # 2 Octet cause of transmission 
		
		
		# \brief List of Quality flags 
class eIEC870QualityFlags(IntEnum):    
		GD          = 0x0000   # Good / Valid 
		IV          = 0x0001   # Invalid 
		NT          = 0x0002   # Not Topical 
		SB          = 0x0004   # Substituted 
		BL          = 0x0008   # Blocked 
		OV          = 0x0010   # Overflow 
		EI          = 0x0020   # Elapsed time  invalid 
		TR          = 0x0040   # Equipement is in Transient State 
		CA          = 0x0080   # Counter was Adjusted 
		CR          = 0x0100   # Carry counter overflow  CY indicated in protocol spec
	
	
	 # \brief Control model configuration 
class eControlModelConfig(IntEnum):    
	STATUS_ONLY                                     = 0       # Status Only For input points In iec104 like M_SP_NA_1... for DNP3 BINARY_INPUT DOUBLE_INPUT...
	DIRECT_OPERATE                                  = 1       # Direct Operate
	SELECT_BEFORE_OPERATE                           = 2       # Select Before Operate 
	

		# \brief counter freeze flags 
class eCounterFreezeFlags(Enum):    
		COUNTER_READ                        = 0   # Counter Read no freeze no reset
		COUNTER_FREEZE              = 1    # Counter freeze without reset value
		COUNTER_FREEZE_WITH_RESET   = 2    # FREEZE Counter value & reset    
		COUNTER_RESET               = 3    # Counter reset 
	

#   Flags for eKPA - Kind of parameter
class eKindofParameter(Enum):        
			PARAMETER_NONE                                     = 0       # Not used
			PARAMETER_THRESHOLDVALUE                           = 1       # Threshold value
			PARAMETER_SMOOTHINGFACTOR                          = 2       # smoothing factor (filter time constant)
			PARAMETER_LOWLIMIT                                 = 3       # smoothing factor (filter time constant) 
			PARAMETER_HIGHLIMIT                                = 4       # smoothing factor (filter time constant) 
		
		
		
		# \brief Cause of Transmission 
class eIEC870COTCause(IntEnum):    
		NOTUSED         = 0
		PERCYC          = 1        # Periodic Cyclic 
		BACK            = 2        # Back ground scan 
		SPONT           = 3        # Spontaneous 
		INIT            = 4        # Initialised 
		REQ             = 5        # Request or Requested 
		ACT             = 6        # Activation 
		ACTCON          = 7        # Activation Confirmation 
		DEACT           = 8        # Deactivation 
		DEACTCON        = 9        # Deactication Confirmation 
		ACTTERM         = 10       # Activation Confirmation 
		RETREM          = 11       # Return information caused by a remote command 
		RETLOC          = 12       # Return information caused by a local command 
		FILETRANFER     = 13       # File transfer 
		INROGEN         = 20       # Interrogated by station interrogation 
		INRO1           = 21       # Interrogated by group 1 interrogation 
		INRO2           = 22       # Interrogated by group 2 interrogation 
		INRO3           = 23       # Interrogated by group 3 interrogation 
		INRO4           = 24       # Interrogated by group 4 interrogation 
		INRO5           = 25       # Interrogated by group 5 interrogation 
		INRO6           = 26       # Interrogated by group 6 interrogation 
		INRO7           = 27       # Interrogated by group 7 interrogation 
		INRO8           = 28       # Interrogated by group 8 interrogation 
		INRO9           = 29       # Interrogated by group 9 interrogation 
		INRO10          = 30       # Interrogated by group 10 interrogation 
		INRO11          = 31       # Interrogated by group 11 interrogation 
		INRO12          = 32       # Interrogated by group 12 interrogation 
		INRO13          = 33       # Interrogated by group 13 interrogation 
		INRO14          = 34       # Interrogated by group 14 interrogation 
		INRO15          = 35       # Interrogated by group 15 interrogation 
		INRO16          = 36       # Interrogated by group 16 interrogation 
		REQCOGEN        = 37       # Requested by general counter request 
		REQCO1          = 38       # Requested by group 1 counter request 
		REQCO2          = 39       # Requested by group 2 counter request 
		REQCO3          = 40       # Requested by group 3 counter request 
		REQCO4          = 41       # Requested by group 4 counter request 
		UNKNOWN_TYPEID  = 44       # Unkown Type Identification 
		UNKNOWN_COT     = 45       # Unknown cause of transmission 
		UNKNOWN_CASDU   = 46       # Unknown common address of ASDU 
		UNKNOWN_IOA     = 47       # Unknown information object address 
	


	
				# \brief  Start events of protection equipment flag 
class eStartEventsofProtFlags(Enum):    
		GS                                      = 0x01       # GS = general start of operation
		SL1                                     = 0x02       # SL1 = start of operation phase L1 
		SL2                                     = 0x04       # SL2 = start of operation phase L2 
		SL3                                     = 0x08       # SL3 = start of operation phase L3 
		SIE                                     = 0x10       # SIE = start of operation IE (earth current) 
		SRD                                     = 0x20       # SRD = start of operation in reverse direction 
	

		# \brief  Packed output circuit information of protection equipment flag 
class ePackedOutputCircuitInfoofProtFlags(Enum):    
		GC                                      = 0x01       # GC = general command to output circuit 
		CL1                                     = 0x02       # CL1 = command to output circuit phase L1 
		CL2                                     = 0x04       # CL2 = command to output circuit phase L2 
		CL3                                     = 0x08       # CL3 = command to output circuit phase L3 
	
	
	
	# List of Command Qualifier 
class eCommandQOCQU(IntEnum):    
		NOADDDEF = 0               #  No Additional Definition 
		SHORTPULSE = 1             # Short Pulse Duration 
		LONGPULSE = 2              # Long Pulse Duration 
		PERSISTANT = 3             # Persistant 
		COMPATIBLE_RANGE_START = 4 # Compatible Range Start 
		COMPATIBLE_RANGE_END = 8   # Compatible Range End 
		PREDEFINED_RANGE_START = 9 # Pre Defined Range Start 
		PREDEFINED_RANGE_END = 15  # Pre Defined Range End 
		PRIVATE_RANGE_START = 16   # Private Range Start 
		PRIVATE_RANGE_END = 31     # Private Range End 
	
		
	# Type Identication List 
class eIEC870TypeID(IntEnum):    
		M_SP_NA_1 = 1      #  Single-point information 
		M_SP_TA_1 = 2      #  Single-point information with time tag - Applicable for IEC 101 only
		M_DP_NA_1 = 3      #  Double-point information 
		M_DP_TA_1 = 4      #  Double-point information with time tag - Applicable for IEC 101 only
		M_ST_NA_1 = 5      #  Step position information 
		M_ST_TA_1 = 6      #  Step position information with time tag - Applicable for IEC 101 only
		M_BO_NA_1 = 7      #  Bitstring of 32 bit  
		M_BO_TA_1 = 8      #  Bitstring of 32 bit with time tag - Applicable for IEC 101 only 
		M_ME_NA_1 = 9      #  Measured value normalized value 
		M_ME_TA_1 = 10     #  Measured value normalized value with time tag   - Applicable for IEC 101 only
		M_ME_NB_1 = 11     #  Measured value scaled value 
		M_ME_TB_1 = 12     #  Measured value scaled value with time tag  - Applicable for IEC 101 only
		M_ME_NC_1 = 13     #  Measured value short floating point value 
		M_ME_TC_1 = 14     #  Measured value short floating point value with time tag - Applicable for IEC 101 only 
		M_IT_NA_1 = 15     #  Integrated totals 
		M_IT_TA_1 = 16     #  Integrated totals with time tag  - Applicable for IEC 101 only
		M_EP_TA_1 = 17     #  Event of protection equipment with time tag  - Applicable for IEC 101 only
		M_EP_TB_1 = 18     #  Packed start events of protection equipment with time tag  - Applicable for IEC 101 only
		M_EP_TC_1 = 19     #  Packed output circuit information of protection equipment with time tag  - Applicable for IEC 101 only
		M_PS_NA_1 = 20     #  Packed single-point information with status change detection  - Applicable for IEC 101 only
		M_ME_ND_1 = 21     #  Measured value normalized value without quality descriptor 
		M_SP_TB_1 = 30     #  Single-point information with time tag CP56Time2a 
		M_DP_TB_1 = 31     #  Double-point information with time tag CP56Time2a 
		M_ST_TB_1 = 32     #  Step position information with time tag CP56Time2a 
		M_BO_TB_1 = 33     #  Bitstring of 32 bit with time tag CP56Time2a 
		M_ME_TD_1 = 34     #  Measured value normalized value with time tag CP56Time2a 
		M_ME_TE_1 = 35     #  Measured value scaled value with time tag CP56Time2a 
		M_ME_TF_1 = 36     #  Measured value short floating point value with time tag CP56Time2a 
		M_IT_TB_1 = 37     #  Integrated totals with time tag CP56Time2a 
		M_EP_TD_1 = 38     #  Event of protection equipment with time tag CP56Time2a 
		M_EP_TE_1 = 39     #  Packed start events of protection equipment with time tag CP56Time2a 
		M_EP_TF_1 = 40     #  Packed output circuit information of protection equipment with time tag CP56Time2a 
		C_SC_NA_1 = 45     #  Single command 
		C_DC_NA_1 = 46     #  Double command 
		C_RC_NA_1 = 47     #  Regulating step command 
		C_SE_NA_1 = 48     #  Set point command normalized value 
		C_SE_NB_1 = 49     #  Set point command scaled value 
		C_SE_NC_1 = 50     #  Set point command short floating point value 
		C_BO_NA_1 = 51     #  Bitstring of 32 bit command 
		C_SC_TA_1 = 58     #  Single command with time tag CP56Time2a 
		C_DC_TA_1 = 59     #  Double command with time tag CP56Time2a 
		C_RC_TA_1 = 60     #  Regulating step command with time tag CP56Time2a 
		C_SE_TA_1 = 61     #  Set point command normalized value with time tag CP56Time2a 
		C_SE_TB_1 = 62     #  Set point command scaled value with time tag CP56Time2a 
		C_SE_TC_1 = 63     #  Set point command short floating point value with time tag CP56Time2a 
		C_BO_TA_1 = 64     #  Bitstring of 32 bit command with time tag CP56Time2a 
		M_EI_NA_1 = 70     #  End of initialization 
		C_IC_NA_1 = 100    #  Interrogation command 
		C_CI_NA_1 = 101    #  Counter interrogation command 
		C_RD_NA_1 = 102    #  Read command 
		C_CS_NA_1 = 103    #  Clock synchronization command 
		C_TS_NA_1 = 104    #  Test command  - Applicable for IEC 101 only
		C_RP_NA_1 = 105    #  Reset process command 
		C_CD_NA_1 = 106    #  Delay acquisition command  - Applicable for IEC 101 only
		C_TS_TA_1 = 107    #  Test command with time tag CP56Time2a 
		P_ME_NA_1 = 110    #  Parameter of measured value normalized value 
		P_ME_NB_1 = 111    #  Parameter of measured value scaled value 
		P_ME_NC_1 = 112    #  Parameter of measured value short floating point value 
		P_AC_NA_1 = 113    #  Parameter activation 
		F_FR_NA_1 = 120    #  File ready 
		F_SR_NA_1 = 121    #  Section ready 
		F_SC_NA_1 = 122    #  Call directory select file call file call section 
		F_LS_NA_1 = 123    #  Last section last segment 
		F_AF_NA_1 = 124    #  Ack file ack section 
		F_SG_NA_1 = 125    #  Segment 
		F_DR_TA_1 = 126    #  Directory 
	





import ctypes
from enum import *

    #  Max Size of Object Name 
APP_OBJNAMESIZE  =       48



     # Application Object State  

class eAppState(Enum):     
         APP_STATE_UNKNOWN           = 0            # A Unknown Object 
         APP_STATE_NEW               = 1            # A Newly create Object 
         APP_STATE_LOADED            = 2            # Object is loaded with a config and ready to run 
         APP_STATE_RUNNING           = 3            # Object is running  
         APP_STATE_STOPPED           = 4            # Object Stopped 
         APP_STATE_FREED             = 5            # Object freed  
     

         #  Time Quality flag 
class eTimeQualityFlags(Enum):    
        TIME_ASSUMED                        = 0# TIME   Assumed, not reported
        TIME_REPORTED                       = 1# TIME   reported. 

    

     # Application Flag 
class eApplicationFlag(IntEnum):     
        APP_SERVER          =   1# Server Application 
        APP_CLIENT          =   2# Client Application 
        APP_SERVERCLIENT    =   3# ServerClient Application 
     

    # Data types 
class eDataTypes(IntEnum):    
        UNSUPPORTED_DATA        = 0# Unsupported                        (0 Bit)                   
        SINGLE_POINT_DATA       = 1# Single Point Data Type             (1 Bit)     
        DOUBLE_POINT_DATA       = 2# Double Point  Data Type            (2 Bits)      
        UNSIGNED_BYTE_DATA      = 3# Unsigned Byte Data Type            (8 Bits)      
        SIGNED_BYTE_DATA        = 4# Signed Byte Data Type              (8 Bits)              
        UNSIGNED_WORD_DATA      = 5# Unsigned Word Data Type            (16 Bits)     
        SIGNED_WORD_DATA        = 6# Signed Word Data Type              (16 Bits)             
        UNSIGNED_DWORD_DATA     = 7# Unsigned Double Word Data Type     (32 Bits)             
        SIGNED_DWORD_DATA       = 8# Signed Double Word Data Type       (32 Bits)    
        UNSIGNED_LWORD_DATA     = 9# Unsigned Long word Data Type       (64 Bits)   
        SIGNED_LWORD_DATA       = 10# Singed long word Data Type         (64 Bits)   
        UNSIGNED_LLWORD_DATA    = 11# Unsigned Long Long Word Data Type  (128 Bits)  
        SIGNED_LLWORD_DATA      = 12# Signed Long Long Word Data Type    (128 Bits)  
        FLOAT32_DATA            = 13# Float 32 Data Type                 (32 Bits)    
        FLOAT64_DATA            = 14# Float 64 Data Type                 (64 Bits)   
        FLOAT128_DATA           = 15# Float 128 Data Type                (128 Bits)  
        VISIBLE_STRING_DATA     = 16# Visible String Data Type           (2040 Bits)  
        MAX_DATATYPES = 17
    

    #   Debug options flag 
class eDebugOptionsFlag(IntEnum):    
        DEBUG_OPTION_NONE                               = 0x0000# No options set 
        DEBUG_OPTION_ERROR                              = 0x0001# Error Messages only 
        DEBUG_OPTION_WARNING                            = 0x0002# Warning Message only      
        DEBUG_OPTION_RX                                 = 0x0004# Rx Messages 
        DEBUG_OPTION_TX                                 = 0x0008# Tx Messages 
        

        
    # SET Time Structure       
class sTargetTimeStamp(ctypes.Structure):
    _fields_ = [("u8Day", ctypes.c_ubyte),              # Day 1 to 31 
        ("u8Month", ctypes.c_ubyte),            # Month 1 to 12 
        ("u16Year",ctypes.c_ushort),            # Year 1970 to 9999 
        ("u8DayoftheWeek", ctypes.c_ubyte),     # 0 = Not Used Day of the week Mon = 1 to Sun = 7 
        ("u8Hour", ctypes.c_ubyte),             # Hour 0 to 23 
        ("u8Minute", ctypes.c_ubyte),           # Minutes 0 to 59 
        ("u8Seconds", ctypes.c_ubyte),          # Seconds 0 to 59 
        ("u16MilliSeconds",ctypes.c_ushort),    # Milliseconds 0 to 999 
        ("u16MicroSeconds",ctypes.c_ushort),    # Micro Seconfs 0 to 999 
        ("i8DSTTime", ctypes.c_byte)]          # -1 DST Unknown, 0 (No DST), 1 to 24 (DST hour) 
    
    




    #  Max Size of IPV4  Address 
MAX_IPV4_ADDRSIZE = 16

    #  Max Size of IPV6  Address 
MAX_IPV6_ADDRSIZE = 40

    #  Max Size of License Path 
MAX_LICENSE_PATH = 242

    #  Max Size of Error Message sent to callback  
MAX_ERROR_MESSAGE = 255

    #  Max Size of Warning Message sent to callback  
MAX_WARNING_MESSAGE = 255


from enum import Enum


 # List of Common Error value returned by API functions 

class eTgtErrorValues(Enum):
          
        EV_NONE                                 =  0        # Everything was ok 
        # Common Error Values                         
        EV_PARAM_NULL                           = -1       # Parameter is NULL 
        EV_INVALID_VALUE                        = -2       # Parameter is Invalid Value         
        EV_OUT_OF_RANGE                         = -3       # Parameter is Out of Range         
        EV_ERRROCODE_NULL                       = -4       # Error Code is NULL 
        EV_NOT_SUPPORTED                        = -5       # Function Not Supported 
        EV_MAX_LIMIT                            = -6       # Exceeds Maximum Limit 
        EV_ALREADY_EXISTED                      = -7       # Already Existed With Same Name 
        EV_NOT_EXISTED                          = -8       # Not Existed 
        # Memory Related Error Values                         
        EV_OUT_OF_MEMORY                        = -100     # Memory NOT Available 
        EV_OS_FREE_MEMORY                       = -101     # Os Free Memory   
        EV_MEMORY_SIZE_INVALID                  = -102     # Size of the Memory is invalid 
        # Socket Related Error Values                         
        EV_SOCKET_SELECT_TIMEOUT                = -300     # Function Wait for Activity Time Out 
        EV_SOCKET_READ_NOT_READY                = -301     # Socket Read not Ready 
        EV_SOCKET_CONNECTION_CLOSED             = -302     # Socket Connection closed 
        EV_SOCKET_NOT_READY                     = -303     # Socket not ready  
        # Linked List Related Error Values                                 
        EV_LIST_IS_EMPTY                        = -350     # Link List is Empty 
        EV_NOT_FOUND                            = -351     # Element NOT Found  
        # File Related Error Values                                 
        EV_NOT_END_OF_FILE                      = -400     # Not Reached End Of File 
        # Serial Related Error Values    
        EV_SERIAL_INITIAL                       = -500     # Serial Initial 
        EV_MESSAGE_INCOMPLETE                   = -501     # Not Fully Transmitted or Received 
        # Message Related Error Values    
        EV_MESSAGE_NOT_AVAILABLE                 = -550     # Message Not AVAILABLE 
        EV_MAC_ADDRESS_NOT_AVAILABLE             = -600     # MAC Address Not AVAILABLE 
        # Modbus Related Error Values 
        EV_MODBUS_START                         =  -1000    # Modbus Error Values Start 
        EV_MODBUS_END                           =  -1499    # Modbus Error Values End 
        # DNP3 Related Error Values 
        EV_DNP3_START                           =  -1500    # DNP3 Error Values Start 
        EV_DNP3_END                             =  -1999    # DNP3 Error Values End 
        # IEC103 Related Error Values 
        EV_IEC103_START                         =  -2000    # IEC103 Error Values Start 
        EV_IEC103_END                           =  -2499    # IEC103 Error Values End 
        # IEC104 Related Error Values 
        EV_IEC104_START                         =  -2500    # IEC104 Error Values Start 
        EV_IEC104_END                           =  -2999    # IEC104 Error Values End 
        # IEC101 Related Error Values 
        EV_IEC101_START                         =  -4500    # IEC104 Error Values Start 
        EV_IEC101_END                           =  -4999    # IEC104 Error Values End 
        # SPABUS Related Error Values 
        EV_SPABUS_START                         =  -5000    # SPABUS Error Values Start 
        EV_SPABUS_END                           =  -5499    # SPABUS Error Values End 

    




import ctypes
import os 
import platform
from .iec101types import *



IEC101_VERSION  = "21.06.018"


system = platform.system()

if 'Windows' in system: 
    # locating the 'iec101x64d.dll' file in the 
    # same directory as this file 
    _file = 'iec101x64d.dll' 

else:
    #linux
    _file = 'libx86_64-iec101.so'

_path = os.path.join(os.path.dirname(__file__), _file)
iec101_lib = ctypes.cdll.LoadLibrary(_path)  


''' /*! \brief          Get Library Version
	\ingroup        Management

	\return         version number of library as a string of char with format AA.BB.CCC

	Example Usage:
	\code
					printf("Version number: %s", IEC101GetLibraryVersion());
	\endcode
*/ '''
#PUBLICAPIPX const Integer8 * PUBLICAPISX IEC101GetLibraryVersion(void);
iec101_lib.IEC101GetLibraryVersion.argtypes = None
iec101_lib.IEC101GetLibraryVersion.restype = ctypes.c_char_p

''' /*! \brief          Get Library Build Time
	\ingroup        Management

	\return         Build time of the library as a string of char. Format "Mmm dd yyyy hh:mm:ss"

	Example Usage:
	\code
					printf("Build Time: %s", IEC101GetLibraryBuildTime());
	\endcode
*/ '''
#PUBLICAPIPX const Integer8 * PUBLICAPISX IEC101GetLibraryBuildTime(void);
iec101_lib.IEC101GetLibraryBuildTime.argtypes = None
iec101_lib.IEC101GetLibraryBuildTime.restype = ctypes.c_char_p

''' /*! \brief          Create a client or server object with call-bacnks for reading, writing and updating data objects
	\ingroup        Management

	\param[in]      psParameters    IEC101 Object Parameters
	\param[out]     pi16ErrorCode     Pointer to a Error Code (if any error occurs)
	\param[out]     ptErrorValue    Pointer to a Error Value (if any error occurs while creating the object)

	\return         Pointer to a new IEC101 object
	\return         NULL if an error occured (errorCode will contain an error code)

	Server Example Usage:
	\code

		Integer16                    i16ErrorCode           = EC_NONE;
		tErrorValue                         tErrorValue          = EV_NONE;
		IEC101Object                           myServer             = NULL;
		struct sIEC101Parameters               sParameters          = {0};

		// Initialize parameters
		sParameters.eAppFlag          = APP_SERVER;      
		sParameters.u32Options          = APP_OPTION_NONE;      // No Options
		sParameters.ptReadCallback    = cbRead;                 // Read Callback
		sParameters.ptWriteCallback   = cbWrite;                // Write Callback           
		sParameters.ptUpdateCallback  = NULL;                   // Update Callback
		sParameters.ptSelectCallback  = cbSelect;                // Select Callback
		sParameters.ptOperateCallback = cbOperate;              // Operate Callback
		sParameters.ptCancelCallback  = cbCancel;               // Cancel Callback
		sParameters.ptFreezeCallback  = cbFreeze;               // freeze Callback
		sParameters.ptDebugCallback   = cbDebug;                // Debug Callback
		sParameters.ptPulseEndActTermCallback = cbpulseend;     // pulse end callback   
		sParameters.ptParameterActCallback = cbParameterAct;    // parameter act Callback
		

		// Create a server
		myServer = IEC101Create(&sParameters, &i16ErrorCode, &tErrorValue);
		if(myServer == NULL)
		{
			printf("\r\n Error iec101Create() failed: %d %d", i16ErrorCode,  tErrorValue);
		
		}   

	\endcode

	Client Example Usage:
	\code

		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;
		IEC101Object                           myClient          = NULL;
		struct sIEC101Parameters               sParameters       = {0};

		// Initialize parameters
		sParameters.eAppFlag          = APP_CLIENT;      // This is a IEC101 master
		sParameters.u32Options        = 0;
		sParameters.ptReadCallback    = NULL;            // Read Callback
		sParameters.ptWriteCallback   = NULL;            // Write Callback           
		sParameters.ptUpdateCallback  = cbUpdate;        // Update Callback
		sParameters.ptSelectCallback  = NULL;            // Select commands
		sParameters.ptOperateCallback = NULL;            // Operate commands
		sParameters.ptCancelCallback  = NULL;            // Cancel commands
		sParameters.ptDebugCallback   = cbDebug;         // Debug Callback
		sParameters.ptPulseEndActTermCallback = NULL;     // pulse end callback
		sParameters.ptClientStatusCallback   = cbClientstatus;         // client connection Callback

			  
		// Create a client
		myClient = IEC101Create(&sParameters, &i16ErrorCode, &tErrorValue);
		if(myClient == NULL)
		{
			printf("\r\n Error iec101Create() failed: %d %d", i16ErrorCode,  tErrorValue);
		
		}

	\endcode
*/ '''
#PUBLICAPIPX IEC101Object PUBLICAPISX IEC101Create(struct sIEC101Parameters *psParameters, Integer16 *pi16ErrorCode, tErrorValue *ptErrorValue);
iec101_lib.IEC101Create.argtypes = [ctypes.POINTER(sIEC101Parameters), ctypes.POINTER(ctypes.c_short), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101Create.restype = ctypes.POINTER(sIEC101AppObject)#IEC101Object

''' /*! \brief          Load the configuration to be used by IEC101 object.
	\ingroup        Management

	\param[in]      myIEC101Obj       IEC101 object 
	\param[in]      psIEC101Config    Pointer to IEC101 Configuration parameters 
	\param[out]     ptErrorValue    Pointer to a Error Value (if any error occurs while creating the object)

	\return         EC_NONE on success
	\return         otherwise error code

	Server Example Usage:
	\code

		Integer16                    i16ErrorCode           = EC_NONE;
		tErrorValue                         tErrorValue          = EV_NONE;
		struct sIEC101ConfigurationParameters  sIEC101Config        = {0};

		sIEC101Config.sServerSet.u8NumberofSerialPortConnections    =   1;

		// Allocate memory for objects
		sIEC101Config.sServerSet.psSerialSet = calloc(sIEC101Config.sServerSet.u8NumberofSerialPortConnections, sizeof(struct sSerialCommunicationSettings ));
		if(sIEC101Config.sServerSet.psSerialSet == NULL)
		{
			printf("\r\nError: Not enough memory to alloc objects");
			break;
		}

	   sIEC101Config.sServerSet.psSerialSet[0].u16SerialPortNumber   =   1;
	   sIEC101Config.sServerSet.psSerialSet[0].eSerialBitRate       =   BITRATE_9600;
	   sIEC101Config.sServerSet.psSerialSet[0].eWordLength          =   WORDLEN_8BITS;
	   sIEC101Config.sServerSet.psSerialSet[0].eSerialParity        =   EVEN;
	   sIEC101Config.sServerSet.psSerialSet[0].eStopBits            =   STOPBIT_1BIT;
	   sIEC101Config.sServerSet.psSerialSet[0].eFlowControl         =  FLOW_NONE;

	   sIEC101Config.sServerSet.psSerialSet[0].sRxTimeParam.u16CharacterTimeout     =   0;
	   sIEC101Config.sServerSet.psSerialSet[0].sRxTimeParam.u16MessageTimeout       =   0;
	   sIEC101Config.sServerSet.psSerialSet[0].sRxTimeParam.u16InterCharacterDelay  =   0;
	   sIEC101Config.sServerSet.psSerialSet[0].sRxTimeParam.u16PostDelay            =   0;
	   sIEC101Config.sServerSet.psSerialSet[0].sRxTimeParam.u16PreDelay             =   0;
	   sIEC101Config.sServerSet.psSerialSet[0].sRxTimeParam.u8CharacterRetries      =   0;
	   sIEC101Config.sServerSet.psSerialSet[0].sRxTimeParam.u8MessageRetries        =   0;

	   sIEC101Config.sServerSet.sServerProtSet.eDataLink                =   BALANCED_MODE;
	   sIEC101Config.sServerSet.sServerProtSet.eCASize                  =   CA_TWO_BYTE;
	   sIEC101Config.sServerSet.sServerProtSet.u8TotalNumberofStations  =   1;
	   sIEC101Config.sServerSet.sServerProtSet.au16CommonAddress[0]     =   1;
	   sIEC101Config.sServerSet.sServerProtSet.au16CommonAddress[1]     =   0;
	   sIEC101Config.sServerSet.sServerProtSet.au16CommonAddress[2]     =   0;
	   sIEC101Config.sServerSet.sServerProtSet.au16CommonAddress[3]     =   0;
	   sIEC101Config.sServerSet.sServerProtSet.au16CommonAddress[4]     =   0;
	   sIEC101Config.sServerSet.sServerProtSet.eCOTsize                 =   COT_TWO_BYTE;
	   sIEC101Config.sServerSet.sServerProtSet.eIOAsize                 =   IOA_TWO_BYTE;
	   sIEC101Config.sServerSet.sServerProtSet.elinkAddrSize            =   DL_TWO_BYTE;
	   
	   sIEC101Config.sServerSet.sServerProtSet.u16DataLinkAddress   =   1;

	   sIEC101Config.sServerSet.sServerProtSet.eNegACK  = FIXED_FRAME_NACK;
	   sIEC101Config.sServerSet.sServerProtSet.ePosACK  =   FIXED_FRAME_ACK;

	   sIEC101Config.sServerSet.sServerProtSet.u16Class1EventBufferSize =   5000;
	   sIEC101Config.sServerSet.sServerProtSet.u16Class2EventBufferSize =   5000;

	   sIEC101Config.sServerSet.sServerProtSet.u8Class1BufferOverFlowPercentage     =   90;
	   sIEC101Config.sServerSet.sServerProtSet.u8Class2BufferOverFlowPercentage     =   90;
	   sIEC101Config.sServerSet.sServerProtSet.u8MaxAPDUSize                        =   253;
	   sIEC101Config.sServerSet.sServerProtSet.u16ShortPulseTime                    =   5000;
	   sIEC101Config.sServerSet.sServerProtSet.u16LongPulseTime                     =   10000;
	   sIEC101Config.sServerSet.sServerProtSet.u32ClockSyncPeriod                   =   0;
	   sIEC101Config.sServerSet.sServerProtSet.bGenerateACTTERMrespond              =   TRUE;


	   sIEC101Config.sServerSet.sServerProtSet.u32BalancedModeTestConnectionSignalInterval  = 60;

	   sIEC101Config.sServerSet.sServerProtSet.bEnableFileTransfer = FALSE;
	   strcpy((char*)sIEC101Config.sServerSet.sServerProtSet.ai8FileTransferDirPath, "\\FileTransferServer");
	   sIEC101Config.sServerSet.sServerProtSet.u16MaxFilesInDirectory    = 10;

	   sIEC101Config.sServerSet.sDebug.u32DebugOptions                              =   ( DEBUG_OPTION_TX | DEBUG_OPTION_RX);
	   //sIEC101Config.sServerSet.sDebug.u32DebugOptions    =   0;

	   sIEC101Config.sServerSet.sServerProtSet.bTransmitSpontMeasuredValue = TRUE;
	   sIEC101Config.sServerSet.sServerProtSet.bTransmitInterrogationMeasuredValue = TRUE;
		sIEC101Config.sServerSet.sServerProtSet.bTransmitBackScanMeasuredValue = TRUE;

		sIEC101Config.sServerSet.sServerProtSet.u8InitialdatabaseQualityFlag = (IV |NT); // 0- good/valid, 1 BIT- iv, 2 BIT-nt,  MAX VALUE -3   
		sIEC101Config.sServerSet.sServerProtSet.bUpdateCheckTimestamp = FALSE; // if it true ,the timestamp change also generate event  during the iec101update 

		sIEC101Config.sServerSet.u16NoofObject           = 2;        // Define number of objects

		// Allocate memory for objects
		sIEC101Config.sServerSet.psIEC101Objects = calloc(sIEC101Config.sServerSet.u16NoofObject, sizeof(struct sIEC101Object));
		if(sIEC101Config.sServerSet.psIEC101Objects == NULL)
		{
			printf("\r\nError: Not enough memory to alloc objects");
			break;
		}

		// Init objects
		//first object detail

		strncpy((char*)sIEC101Config.sServerSet.psIEC101Objects[0].ai8Name,"M_ME_TF_1",APP_OBJNAMESIZE);
		sIEC101Config.sServerSet.psIEC101Objects[0].eTypeID     = M_ME_TF_1;
		sIEC101Config.sServerSet.psIEC101Objects[0].u32IOA          = 100;
		sIEC101Config.sServerSet.psIEC101Objects[0].eIntroCOT       = INRO1;
		sIEC101Config.sServerSet.psIEC101Objects[0].u16Range        = 10;
		sIEC101Config.sServerSet.psIEC101Objects[0].eControlModel   =   STATUS_ONLY;
		sIEC101Config.sServerSet.psIEC101Objects[0].u32SBOTimeOut   =   0;
		sIEC101Config.sServerSet.psIEC101Objects[0].eClass          =   IEC_CLASS1;
		sIEC101Config.sServerSet.psIEC101Objects[0].u16CommonAddress    =   1;

		//Second object detail
		strncpy((char*)sIEC101Config.sServerSet.psIEC101Objects[1].ai8Name,"C_SE_TC_1",APP_OBJNAMESIZE);
		sIEC101Config.sServerSet.psIEC101Objects[1].eTypeID     =  C_SE_TC_1;
		sIEC101Config.sServerSet.psIEC101Objects[1].u32IOA          = 100;
		sIEC101Config.sServerSet.psIEC101Objects[1].eIntroCOT       = NOTUSED;
		sIEC101Config.sServerSet.psIEC101Objects[1].u16Range        = 10;
		sIEC101Config.sServerSet.psIEC101Objects[1].eControlModel  = DIRECT_OPERATE;
		sIEC101Config.sServerSet.psIEC101Objects[1].u32SBOTimeOut   = 0;
		sIEC101Config.sServerSet.psIEC101Objects[1].eClass          =   IEC_NO_CLASS;
		sIEC101Config.sServerSet.psIEC101Objects[1].u16CommonAddress    =   1;

   
		// Load configuration
	   i16ErrorCode = IEC101LoadConfiguration(myServer, &sIEC101Config, &tErrorValue);
	   if(i16ErrorCode != EC_NONE)
	   {
			printf("\r\nError: IEC101LoadConfiguration() failed: %d - %s, %d - %s ", i16ErrorCode, errorcodestring(i16ErrorCode),  tErrorValue , errorvaluestring(tErrorValue));
			break;
		}


	\endcode

	Client Example Usage:
	\code
		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;
		struct sIEC101ConfigurationParameters  sIEC101Config   = {0};

		sIEC101Config.sClientSet.u8NoofClient   =   1;
		sIEC101Config.sClientSet.psClientObjects    =   calloc(sIEC101Config.sClientSet.u8NoofClient, sizeof(struct sIECClientObject ));

		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.u16SerialPortNumber   =   2;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.eSerialBitRate       =   BITRATE_9600;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.eWordLength          =   WORDLEN_8BITS;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.eSerialParity        =   EVEN;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.eStopBits            =   STOPBIT_1BIT;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.eFlowControl =  FLOW_NONE;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.sRxTimeParam.u16CharacterTimeout     =   0;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.sRxTimeParam.u16MessageTimeout       =   0;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.sRxTimeParam.u16InterCharacterDelay  =   0;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.sRxTimeParam.u16PostDelay            =   0;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.sRxTimeParam.u16PreDelay             =   0;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.sRxTimeParam.u8CharacterRetries      =   0;
		sIEC101Config.sClientSet.psClientObjects[0].sSerialSet.sRxTimeParam.u8MessageRetries        =   0;

		sIEC101Config.sClientSet.eLink  =   UNBALANCED_MODE;
		
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.eCASize  =   CA_TWO_BYTE;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.eCOTsize =   COT_TWO_BYTE;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u8OriginatorAddress  =   1;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.eIOAsize =   IOA_TWO_BYTE;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.elinkAddrSize    =   DL_TWO_BYTE;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u8TotalNumberofStations  =   1;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.au16CommonAddress[0] =   1;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.au16CommonAddress[1]     =   0;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.au16CommonAddress[2]     =   0;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.au16CommonAddress[3]     =   0;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.au16CommonAddress[4]     =   0;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u16DataLinkAddress   =   1;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32LinkLayerTimeout  =   1000;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32PollInterval  =   1000;

		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32GeneralInterrogationInterval  =   0;    // in sec if 0 , gi will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group1InterrogationInterval   =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group2InterrogationInterval   =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group3InterrogationInterval   =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group4InterrogationInterval   =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group5InterrogationInterval   =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group6InterrogationInterval   =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group7InterrogationInterval   =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group8InterrogationInterval   =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group9InterrogationInterval   =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group10InterrogationInterval  =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group11InterrogationInterval  =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group12InterrogationInterval  =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group13InterrogationInterval  =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group14InterrogationInterval  =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group15InterrogationInterval  =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group16InterrogationInterval  =   0;    // in sec if 0 , group 1 interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32CounterInterrogationInterval  =   0;    // in sec if 0 , ci will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group1CounterInterrogationInterval    =   0;    // in sec if 0 , group 1 counter interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group2CounterInterrogationInterval    =   0;    // in sec if 0 , group 1 counter interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group3CounterInterrogationInterval    =   0;    // in sec if 0 , group 1 counter interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32Group4CounterInterrogationInterval    =   0;    // in sec if 0 , group 1 counter interrogation will not send in particular interval
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32ClockSyncInterval =   0;              // in sec if 0 , clock sync, will not send in particular interval 


		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32CommandTimeout    =   5000;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.bCommandResponseActtermUsed  =   TRUE;
		sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.u32FileTransferTimeout           = 1000000;
		strcpy((char*)sIEC101Config.sClientSet.psClientObjects[0].sClientProtSet.ai8FileTransferDirPath, "D:\\FileTest");

		//sIEC101Config.sClientSet.sDebug.u32DebugOptions   =   0;
		sIEC101Config.sClientSet.sDebug.u32DebugOptions                             =   ( DEBUG_OPTION_TX | DEBUG_OPTION_RX);

		sIEC101Config.sClientSet.psClientObjects[0].u16NoofObject           = 2;        // Define number of objects

		// Allocate memory for objects
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects = calloc(sIEC101Config.sClientSet.psClientObjects[0].u16NoofObject, sizeof(struct sIEC101Object));
		if(sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects == NULL)
		{
			printf("\r\nError: Not enough memory to alloc objects");
			break;
		}

		strcpy((char*)sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[0].ai8Name,"M_ME_TF_1");
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[0].eTypeID      = M_ME_TF_1;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[0].u32IOA           = 100;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[0].eIntroCOT    = INRO1;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[0].u16Range     = 10;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[0].u32CyclicTransTime   =   0;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[0].eControlModel  =  STATUS_ONLY ;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[0].u32SBOTimeOut    = 0;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[0].eClass           =   IEC_CLASS1;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[0].u16CommonAddress =   1;

		strncpy((char*)sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[1].ai8Name,"C_SE_TC_1",APP_OBJNAMESIZE);
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[1].eTypeID      = C_SE_TC_1;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[1].u32IOA           = 100;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[1].eIntroCOT    = NOTUSED;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[1].u16Range     = 10;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[1].u32CyclicTransTime   =   0;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[1].eControlModel  = DIRECT_OPERATE;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[1].u32SBOTimeOut    = 0;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[1].eClass           =   IEC_NO_CLASS;
		sIEC101Config.sClientSet.psClientObjects[0].psIEC101Objects[1].u16CommonAddress =   1;


		// Load configuration
		i16ErrorCode = IEC101LoadConfiguration(myClient, &sIEC101Config, &tErrorValue);
		if(i16ErrorCode != EC_NONE)
		{
			printf("\r\nError: IEC101LoadConfiguration() failed:   %d - %s, %d - %s ", i16ErrorCode, errorcodestring(i16ErrorCode),  tErrorValue , errorvaluestring(tErrorValue));
			break;
		}


	\endcode
*/ '''
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101LoadConfiguration(IEC101Object myIEC101Obj, struct sIEC101ConfigurationParameters *psIEC101Config, tErrorValue *ptErrorValue);
iec101_lib.IEC101LoadConfiguration.argtypes = [IEC101Object, ctypes.POINTER(sIEC101ConfigurationParameters), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101LoadConfiguration.restype = ctypes.c_short
''' /*! \brief          Start IEC101 object communication
	\ingroup        Management

	\param[in]      myIEC101Obj     IEC101 object to Start
	\param[out]     ptErrorValue    Pointer to a Error Value (if any error occurs while creating the object)

	\return         EC_NONE on success
	\return         otherwise error code

	Server Example Usage:
	\code

		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;

		// Start server
		i16ErrorCode = IEC101Start(myServer, &tErrorValue);  //Start myServer
		if(i16ErrorCode != EC_NONE)
		{  
			printf("\r\n Error IEC101Start() failed: %i %i", i16ErrorCode,    tErrorValue);
			
		}

	\endcode 

	Client Example Usage:
	\code

		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;

		// Start Client
		i16ErrorCode = IEC101Start(myClient, &tErrorValue);  //Start myClient
		if(i16ErrorCode != EC_NONE)
		{  
			printf("\r\n Error IEC101Start() failed: %i %i", i16ErrorCode,  tErrorValue);
			break;
		}            

	\endcode

	
*/ '''
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101Start(IEC101Object myIEC101Obj, tErrorValue *ptErrorValue);
iec101_lib.IEC101Start.argtypes = [IEC101Object, ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101Start.restype = ctypes.c_short

''' /*! \brief           Set IEC101 debug options.
	 \ingroup         Management
 
	  \param[in]      myIEC101Obj         IEC101 object to Get Type and Size
	  \param[in]      psDebugParams       Pointer to debug parameters
	  \param[out]     ptErrorValue        Pointer to a Error Value (if any error occurs while creating the object)
 
	  \return         EC_NONE on success
	  \return         otherwise error code
 
	  Example Usage:
	  \code
					
		Integer16                  i16ErrorCode        = EC_NONE;
		tErrorValue                       tErrorValue       = EV_NONE;
					  
		struct sIEC101DebugParameters       sDebugParams    = {0};
 
		// Set the debug option to error, tx and rx data 
		sDebugParams.u32DebugOptions   = DEBUG_OPTION_ERROR | DEBUG_OPTION_TX | DEBUG_OPTION_RX;
 
		//update debug option
		i16ErrorCode = IEC101SetDebugOptions(myIEC101Obj, &sDebugParams, &tErrorValue);
		if(i16ErrorCode != EC_NONE)
		{
			printf("Set debug options IEC101 has failed: %i %i", i16ErrorCode, tErrorValue);
		}

	  \endcode 

	 
  */ '''
 #PUBLICAPIPX Integer16 PUBLICAPISX IEC101SetDebugOptions(IEC101Object myIEC101Obj, struct sIEC101DebugParameters *psDebugParams, tErrorValue *ptErrorValue);            
iec101_lib.IEC101SetDebugOptions.argtypes = [IEC101Object, ctypes.POINTER(sIEC101DebugParameters), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101SetDebugOptions.restype = ctypes.c_short

''' /*! \brief          Stop IEC101 object communication
	\ingroup        Management

	\param[in]      myIEC101Obj     IEC101 object to Stop
	\param[out]     ptErrorValue    Pointer to a Error Value (if any error occurs while creating the object)

	\return         EC_NONE on success
	\return         otherwise error code

	Server Example Usage:
	\code

		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;

		// Stop server
		i16ErrorCode = IEC101Stop(myServer, &tErrorValue);  //Stop myServer
		if(i16ErrorCode != EC_NONE)
		{  
			printf("\r\n Error IEC101Stop() failed: %i %i", i16ErrorCode, tErrorValue);
			
		}

	\endcode 

	Client Example Usage:
	\code

		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;

		// Stop Client
		i16ErrorCode = IEC101Stop(myClient, &tErrorValue);  //Stop myClient
		if(i16ErrorCode != EC_NONE)
		{  
			printf("\r\n Error IEC101Stop() failed: %i %i", i16ErrorCode,  tErrorValue);
			break;
		}            

	\endcode

*/ '''
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101Stop(IEC101Object myIEC101Obj,tErrorValue *ptErrorValue);
iec101_lib.IEC101Stop.argtypes = [IEC101Object, ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101Stop.restype = ctypes.c_short
''' /*! \brief          Free memory used by IEC101 object.
	\ingroup        Management

	\param[in]      myIEC101Obj     IEC101 object to free
	\param[out]     ptErrorValue    Pointer to a Error Value (if any error occurs while creating the object)

	\return         EC_NONE on success
	\return         otherwise error code

	Server Example Usage:
	\code

		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;

		// free server
		i16ErrorCode = IEC101Free(myServer, &tErrorValue);  //free myServer
		if(i16ErrorCode != EC_NONE)
		{  
			printf("\r\n Error IEC101free() failed: %i %i", i16ErrorCode, tErrorValue);
			
		}

	\endcode 

	Client Example Usage:
	\code

		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;

		// free Client
		i16ErrorCode = IEC101Free(myClient, &tErrorValue);  //free myClient
		if(i16ErrorCode != EC_NONE)
		{  
			printf("\r\n Error IEC101free() failed: %i %i", i16ErrorCode,  tErrorValue);
			break;
		}            

	\endcode

*/ '''
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101Free(IEC101Object myIEC101Obj, tErrorValue *ptErrorValue);
iec101_lib.IEC101Free.argtypes = [IEC101Object, ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101Free.restype = ctypes.c_short

''' /*!\brief           Update IEC101 data attribute ID to the New Value. 
	\ingroup        Management

	\param[in]      myIEC101Obj     IEC101 object to Update
	\param[in]      bCreateEvent    Generate event or not, if false, only update the database, not genearate event
	\param[in]      psDAID          Pointer to IEC101 Data Attribute ID
	\param[in]      psNewValue      Pointer to IEC101 Data Attribute Data
	\param[in]      u16Count        Number of IEC101 Data attribute ID and Data attribute data to be updated simultaneously
	\param[out]     ptErrorValue    Pointer to a Error Value (if any error occurs while creating the object)

	\return         EC_NONE on success
	\return         otherwise error code

	Server Example Usage:
	\code

		
		Integer16                    i16ErrorCode       = EC_NONE;
		tErrorValue                         tErrorValue      = EV_NONE;
	
		struct sIEC101DataAttributeID *psDAID                   = NULL; //update dataaddtribute
		struct sIEC101DataAttributeData *psNewValue             = NULL; //updtae new value
		unsigned int uiCount;

		Unsigned8   u8Data                      = 1;
		Float32 f32Data                         = -10;

		// update parameters
		uiCount     =   2;
		psDAID      =   calloc(uiCount,sizeof(struct sIEC101DataAttributeID));
		psNewValue  =   calloc(uiCount,sizeof(struct sIEC101DataAttributeData));
	
	
		psDAID[0].u16SerialPortNumber                =   1;
		psDAID[0].u16DataLinkAddress                =   1;
		psDAID[0].eTypeID                           =   M_SP_NA_1;
		psDAID[0].u32IOA                            =   5006;
		psDAID[0].u16CommonAddress                      =   1;
		psDAID[0].pvUserData                        =   NULL;
		psNewValue[0].tQuality                      =   GD;
		//current date 11/8/2012
		psNewValue[0].sTimeStamp.u8Day              =   8;
		psNewValue[0].sTimeStamp.u8Month            =   11;
		psNewValue[0].sTimeStamp.u16Year            =   2012;
	
		//time 13.35.0
		psNewValue[0].sTimeStamp.u8Hour             =   13;
		psNewValue[0].sTimeStamp.u8Minute           =   36;
		psNewValue[0].sTimeStamp.u8Seconds          =   0;
		psNewValue[0].sTimeStamp.u16MilliSeconds    =   0;
		psNewValue[0].sTimeStamp.u16MicroSeconds    =   0;
		psNewValue[0].sTimeStamp.i8DSTTime          =   0; //No Day light saving time
		psNewValue[0].sTimeStamp.u8DayoftheWeek     =   4;
	
		psNewValue[0].pvData                        =   &u8Data;
		psNewValue[0].eDataType                     =   SINGLE_POINT_DATA;
		psNewValue[0].eDataSize                     =   DOUBLE_POINT_SIZE;
	
		psDAID[1].u16SerialPortNumber                =   1;
		psDAID[1].u16DataLinkAddress                =   1;
		psDAID[1].eTypeID                           =   M_ME_TF_1;
		psDAID[1].u32IOA                            =   7006L;
		psDAID[1].u16CommonAddress                  =   1;
		psDAID[1].pvUserData                        =   NULL;
		psNewValue[1].tQuality                      =   GD;
		//current date 11/8/2012
		psNewValue[1].sTimeStamp.u8Day              =   8;
		psNewValue[1].sTimeStamp.u8Month            =   11;
		psNewValue[1].sTimeStamp.u16Year            =   2012;
	
		//time 13.35.0
		psNewValue[1].sTimeStamp.u8Hour             =   13;
		psNewValue[1].sTimeStamp.u8Minute           =   36;
		psNewValue[1].sTimeStamp.u8Seconds          =   0;
		psNewValue[1].sTimeStamp.u16MilliSeconds    =   0;
		psNewValue[1].sTimeStamp.u16MicroSeconds    =   0;
		psNewValue[1].sTimeStamp.i8DSTTime          =   0; //No Day light saving time
		psNewValue[1].sTimeStamp.u8DayoftheWeek     =   4;
	
		psNewValue[1].pvData                        =   &f32Data;
		psNewValue[1].eDataType                     =   FLOAT32_DATA;
		psNewValue[1].eDataSize                     =   FLOAT32_SIZE;

		// Update server
		i16ErrorCode = IEC101Update(myServer,psDAID,psNewValue,uiCount,&tErrorValue);  //Update myServer
		if(i16ErrorCode != EC_NONE)
		{
			printf("\r\nError: IEC101Update() failed:  %i %i", i16ErrorCode, tErrorValue);
		}

	\endcode 

*/ '''            
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101Update(IEC101Object myIEC101Obj, Boolean bCreateEvent, struct sIEC101DataAttributeID *psDAID, struct sIEC101DataAttributeData *psNewValue, Unsigned16 u16Count, tErrorValue *ptErrorValue);
bCreateEvent = ctypes.c_bool
u16Count = ctypes.c_ushort
iec101_lib.IEC101Update.argtypes = [IEC101Object, bCreateEvent, ctypes.POINTER(sIEC101DataAttributeID), ctypes.POINTER(sIEC101DataAttributeData), u16Count, ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101Update.restype = ctypes.c_short
''' /*!\brief           Read a value to a given Object ID. 
	\ingroup        Management

	\param[in]      myIEC101Obj       IEC101 object 
	\param[in]      psDAID          Pointer to IEC101 DataAttributeID structure (or compatable) that idendifies the point that is to be read
	\param[in]      psReturnedValue Pointer to Object Data structure that hold the returned vaule
	\param[out]     ptErrorValue    Pointer to a Error Value (if any error occurs while reading the object)

	\return         EC_NONE on success
	\return         otherwise error code

	Client Example Usage:
	\code
	
		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;
	
		struct sIEC101DataAttributeID sDAID;
		struct sIEC101DataAttributeData sReturnedValue;
	
		sDAID.u16SerialPortNumber                =   1;
		sDAID.u16DataLinkAddress                =   1;
		sDAID.u16CommonAddress                  =   1;
		sDAID.eTypeID               =   M_SP_NA_1;
		sDAID.u16DataLinkAddress    =   1;
		sDAID.u32IOA                =   8006;
				
		i16ErrorCode =    IEC101Read(myClient,&sDAID, &sReturnedValue,&tErrorValue);
		if(i16ErrorCode != EC_NONE)
		{
			printf("\r\nError: IEC101Read() failed:  %i %i", i16ErrorCode, tErrorValue);
		}
		
	\endcode            
*/ '''
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101Read(IEC101Object myIEC101Obj, struct sIEC101DataAttributeID *psDAID, struct sIEC101DataAttributeData *psReturnedValue, tErrorValue *ptErrorValue);
iec101_lib.IEC101Read.argtypes = [IEC101Object, ctypes.POINTER(sIEC101DataAttributeID), ctypes.POINTER(sIEC101DataAttributeData), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101Read.restype = ctypes.c_short
''' /*!\brief           IEC101Client - send clock sync, General Interrogation, counter interrogation command. 
	\ingroup        Management

	\param[in]      myIEC101Obj       IEC101 object 
	\param[in]      eCounterFreeze    enum eCounterFreezeFlags
	\param[in]      psDAID            Pointer to IEC101_DataAttributeID structure (or compatable) that idendifies the point that is to be written
	\param[in]      psWriteValue      Pointer to Object Data structure that hold the new vaule of the struct sIEC101DataAttributeData 
	\param[in]      ptWriteParams     Pointer to struct sIEC101WriteParameters 
	\param[out]     ptErrorValue      Pointer to a Error Value 

	\return         EC_NONE on success
	\return         otherwise error code
	

Client Example Usage:
\code

	Integer16                    i16ErrorCode        = EC_NONE;
	tErrorValue                         tErrorValue       = EV_NONE;

	struct sIEC101DataAttributeID sDAID;
	struct sIEC101DataAttributeData sWriteValue;

	sDAID.u16SerialPortNumber                =   1;
	sDAID.u16DataLinkAddress                =   1;
	sDAID.u16CommonAddress                  =   1;
	sDAID.eTypeID               =   C_IC_NA_1;
	sDAID.u16DataLinkAddress    =   1;
	
			
	i16ErrorCode =    IEC101Write(myClient,&sDAID, &sWriteValue,&tErrorValue);
	if(i16ErrorCode != EC_NONE)
	{
		printf("\r\nError: IEC101Write() failed:  %i %i", i16ErrorCode, tErrorValue);
	}

 \endcode

*/ '''    
        
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101Write(IEC101Object myIEC101Obj, enum eCounterFreezeFlags eCounterFreeze, struct sIEC101DataAttributeID *psDAID, struct sIEC101DataAttributeData *psWriteValue, struct sIEC101WriteParameters *ptWriteParams, tErrorValue *ptErrorValue);
eCounterFreeze = ctypes.c_int
iec101_lib.IEC101Write.argtypes = [IEC101Object, eCounterFreeze, ctypes.POINTER(sIEC101DataAttributeID), ctypes.POINTER(sIEC101DataAttributeData), ctypes.POINTER(sIEC101WriteParameters), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101Write.restype = ctypes.c_short
''' /*!\brief           IEC101Client Select a given control Data object.             
	\ingroup        Management

	\param[in]      myIEC101Obj       IEC101 object 
	\param[in]      psDAID          Pointer to IEC101 Data Attribute ID of control that is to be Selected
	\param[in]      psSelectValue   Pointer to IEC101 Data Attribute Data (The value the control is to be set)
	\param[in]      psSelectParams  Pointer to IEC101 Data Attribute Parameters (Quality Paramters)
	\param[out]     ptErrorValue    Pointer to a Error Value 

	\return         EC_NONE on success
	\return         otherwise error code

	Client Example Usage:
	\code

		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;

		Float32             f32value        =   0;
		struct sIEC101DataAttributeID sDAID;
		struct sIEC101DataAttributeData sSelectValue;
		struct sIEC101CommandParameters sSelectParams;
		
		sDAID.u16SerialPortNumber                =   1;
		sDAID.u16DataLinkAddress                =   1;
		sDAID.u16CommonAddress                  =   1;
		sDAID.eTypeID               =   C_SE_TC_1;
		sDAID.u16DataLinkAddress    =   1;
		sDAID.u32IOA                =   8006;

		f32value                    =   -1.2345;
		sSelectValue.eDataSize      =   FLOAT32_SIZE;
		sSelectValue.eDataType      =   FLOAT32_DATA;
		sSelectValue.pvData         =   &f32value;

		sSelectParams.eQOCQU        =   NOADDDEF;
		memset(&sSelectValue.sTimeStamp, 0, sizeof(struct sTargetTimeStamp));

		//current date 11/8/2012
		sSelectValue.sTimeStamp.u8Day               =   8;
		sSelectValue.sTimeStamp.u8Month             =   11;
		sSelectValue.sTimeStamp.u16Year             =   2012;

		//time 13.35.0
		sSelectValue.sTimeStamp.u8Hour              =   13;
		sSelectValue.sTimeStamp.u8Minute            =   36;
		sSelectValue.sTimeStamp.u8Seconds           =   0;
		sSelectValue.sTimeStamp.u16MilliSeconds     =   0;
		sSelectValue.sTimeStamp.u16MicroSeconds     =   0;
		sSelectValue.sTimeStamp.i8DSTTime           =   0; //No Day light saving time
		sSelectValue.sTimeStamp.u8DayoftheWeek      =   4;
		
		
		i16ErrorCode =    IEC101Select(myClient,&sDAID, &sSelectValue, &sSelectParams,&tErrorValue);
		if(i16ErrorCode != EC_NONE)
		{
			printf("\r\nError: IEC101Select() failed:  %i %i", i16ErrorCode, tErrorValue);
		}

	\endcode
*/ '''            
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101Select(IEC101Object myIEC101Obj, struct sIEC101DataAttributeID *psDAID, struct sIEC101DataAttributeData *psSelectValue, struct sIEC101CommandParameters *psSelectParams , tErrorValue *ptErrorValue);
iec101_lib.IEC101Select.argtypes = [IEC101Object, ctypes.POINTER(sIEC101DataAttributeID), ctypes.POINTER(sIEC101DataAttributeData), ctypes.POINTER(sIEC101CommandParameters), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101Select.restype = ctypes.c_short
''' /*!\brief           Send an Operate command on given control Data object. 
	\ingroup        Management

	\param[in]      myIEC101Obj       IEC101 object 
	\param[in]      psDAID          Pointer to IEC101 Data Attribute ID of control that is to be Operated
	\param[in]      psOperateValue  Pointer to IEC101 Data Attribute Data (The value the control is to be set )
	\param[in]      psOperateParams Pointer to IEC101 Data Attribute Parameters (Quality Paramters)
	\param[out]     ptErrorValue    Pointer to a Error Value 

	\return         EC_NONE on success
	\return         otherwise error code

	Client Example Usage:
	\code
	
		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;

		Float32             f32value        =   0;
		struct sIEC101DataAttributeID sDAID;
		struct sIEC101DataAttributeData sOperateValue;
		struct sIEC101CommandParameters sOperateParams;

		sDAID.u16SerialPortNumber                =   1;
		sDAID.u16DataLinkAddress                =   1;
		sDAID.u16CommonAddress                  =   1;
		sDAID.eTypeID               =   C_SE_TC_1;
		sDAID.u16DataLinkAddress    =   1;
		sDAID.u32IOA                =   8006;

		f32value                    =   -1.2345;
		sOperateValue.eDataSize     =   FLOAT32_SIZE;
		sOperateValue.eDataType     =   FLOAT32_DATA;
		sOperateValue.pvData            =   &f32value;

		sOperateParams.eQOCQU       =   NOADDDEF;
		memset(&sOperateValue.sTimeStamp, 0, sizeof(struct sTargetTimeStamp));

		//current date 11/8/2012
		sOperateValue.sTimeStamp.u8Day              =   8;
		sOperateValue.sTimeStamp.u8Month                =   11;
		sOperateValue.sTimeStamp.u16Year                =   2012;

		//time 13.35.0
		sOperateValue.sTimeStamp.u8Hour             =   13;
		sOperateValue.sTimeStamp.u8Minute           =   36;
		sOperateValue.sTimeStamp.u8Seconds          =   0;
		sOperateValue.sTimeStamp.u16MilliSeconds        =   0;
		sOperateValue.sTimeStamp.u16MicroSeconds        =   0;
		sOperateValue.sTimeStamp.i8DSTTime          =   0; //No Day light saving time
		sOperateValue.sTimeStamp.u8DayoftheWeek     =   4;
		
		
		i16ErrorCode =    IEC101Operate(myClient,&sDAID, &sOperateValue, &sOperateParams,&tErrorValue);
		if(i16ErrorCode != EC_NONE)
		{
			printf("\r\nError: IEC101Operate() failed:  %i %i", i16ErrorCode, tErrorValue);
		}
		
	\endcode            
*/ '''             
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101Operate(IEC101Object myIEC101Obj, struct sIEC101DataAttributeID *psDAID, struct sIEC101DataAttributeData *psOperateValue, struct sIEC101CommandParameters *psOperateParams, tErrorValue *ptErrorValue);
iec101_lib.IEC101Operate.argtypes = [IEC101Object, ctypes.POINTER(sIEC101DataAttributeID), ctypes.POINTER(sIEC101DataAttributeData), ctypes.POINTER(sIEC101CommandParameters), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101Operate.restype = ctypes.c_short
''' /*!\brief           IEC101Client SelectBeforeOperate a given control Data object.             
	\ingroup        Management

	\param[in]      myIEC101Obj       IEC101 object 
	\param[in]      psDAID          Pointer to IEC101 Data Attribute ID of control that is to be Selected
	\param[in]      psSelectValue   Pointer to IEC101 Data Attribute Data (The value the control is to be set)
	\param[in]      psSelectParams  Pointer to IEC101 Data Attribute Parameters (Quality Paramters)
	\param[out]     ptErrorValue    Pointer to a Error Value 

	\return         EC_NONE on success
	\return         otherwise error code

	Client Example Usage:
	\code

		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;

		Float32             f32value        =   0;
		struct sIEC101DataAttributeID sDAID;
		struct sIEC101DataAttributeData sSelectValue;
		struct sIEC101CommandParameters sSelectParams;
		
		sDAID.u16SerialPortNumber                =   1;
		sDAID.u16DataLinkAddress                =   1;
		sDAID.u16CommonAddress                  =   1;
		sDAID.eTypeID               =   C_SE_TC_1;
		sDAID.u16DataLinkAddress    =   1;
		sDAID.u32IOA                =   8006;

		f32value                    =   -1.2345;
		sSelectValue.eDataSize      =   FLOAT32_SIZE;
		sSelectValue.eDataType      =   FLOAT32_DATA;
		sSelectValue.pvData         =   &f32value;

		sSelectParams.eQOCQU        =   NOADDDEF;
		memset(&sSelectValue.sTimeStamp, 0, sizeof(struct sTargetTimeStamp));

		//current date 11/8/2012
		sSelectValue.sTimeStamp.u8Day               =   8;
		sSelectValue.sTimeStamp.u8Month             =   11;
		sSelectValue.sTimeStamp.u16Year             =   2012;

		//time 13.35.0
		sSelectValue.sTimeStamp.u8Hour              =   13;
		sSelectValue.sTimeStamp.u8Minute            =   36;
		sSelectValue.sTimeStamp.u8Seconds           =   0;
		sSelectValue.sTimeStamp.u16MilliSeconds     =   0;
		sSelectValue.sTimeStamp.u16MicroSeconds     =   0;
		sSelectValue.sTimeStamp.i8DSTTime           =   0; //No Day light saving time
		sSelectValue.sTimeStamp.u8DayoftheWeek      =   4;
		
		
		i16ErrorCode =    IEC101Select(myClient,&sDAID, &sSelectValue, &sSelectParams,&tErrorValue);
		if(i16ErrorCode != EC_NONE)
		{
			printf("\r\nError: IEC101Select() failed:  %i %i", i16ErrorCode, tErrorValue);
		}

	\endcode
*/ '''            
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101SelectBeforeOperate(IEC101Object myIEC101Obj, struct sIEC101DataAttributeID *psDAID, struct sIEC101DataAttributeData *psSelectValue, struct sIEC101CommandParameters *psSelectParams , tErrorValue *ptErrorValue);
iec101_lib.IEC101SelectBeforeOperate.argtypes = [IEC101Object, ctypes.POINTER(sIEC101DataAttributeID), ctypes.POINTER(sIEC101DataAttributeData), ctypes.POINTER(sIEC101CommandParameters), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101SelectBeforeOperate.restype = ctypes.c_short
''' /*!\brief           Cancel current command on given control Data object. 
	\ingroup        Management

	
	\param[in]      myIEC101Obj     IEC101 object 
	\param[in]      eOperation      Select/Operate to cancel enum eOperationFlag
	\param[in]      psDAID          Pointer to IEC101 Data Attribute ID of control that is to be canceled
	\param[in]      psCancelValue   Pointer to IEC101 Data Attribute Data (The value the control is to be set to)
	\param[in]      psCancelParams  Pointer to struct sIEC101CommandParameters (Quality Paramters)
	\param[out]     ptErrorValue    Pointer to a Error Value 

	\return         EC_NONE on success
	\return         otherwise error code

	Client Example Usage:
	\code

		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;

		Float32             f32value        =   0;
		struct sIEC101DataAttributeID sDAID;
		struct sIEC101DataAttributeData sCancelValue;
		struct sIEC101CommandParameters sCancelParams;

		sDAID.u16SerialPortNumber                =   1;
		sDAID.u16DataLinkAddress                =   1;
		sDAID.u16CommonAddress                  =   1;
		sDAID.eTypeID               =   C_SE_TC_1;
		sDAID.u16DataLinkAddress    =   1;
		sDAID.u32IOA                =   8006;

		f32value                    =   -1.2345;
		sCancelValue.eDataSize      =   FLOAT32_SIZE;
		sCancelValue.eDataType      =   FLOAT32_DATA;
		sCancelValue.pvData         =   &f32value;

		sCancelParams.eQOCQU        =   NOADDDEF;
		memset(&sCancelValue.sTimeStamp, 0, sizeof(struct sTargetTimeStamp));

		//current date 11/8/2012
		sCancelValue.sTimeStamp.u8Day               =   8;
		sCancelValue.sTimeStamp.u8Month             =   11;
		sCancelValue.sTimeStamp.u16Year             =   2012;

		//time 13.35.0
		sCancelValue.sTimeStamp.u8Hour              =   13;
		sCancelValue.sTimeStamp.u8Minute            =   36;
		sCancelValue.sTimeStamp.u8Seconds           =   0;
		sCancelValue.sTimeStamp.u16MilliSeconds     =   0;
		sCancelValue.sTimeStamp.u16MicroSeconds     =   0;
		sCancelValue.sTimeStamp.i8DSTTime           =   0; //No Day light saving time
		sCancelValue.sTimeStamp.u8DayoftheWeek      =   4;
		
		
		i16ErrorCode =    IEC101Cancel(OPERATE, myClient,&sDAID, &sCancelValue, &sCancelParams,&tErrorValue);
		if(i16ErrorCode != EC_NONE)
		{
			printf("\r\nError: IEC101Cancel() failed:  %i %i", i16ErrorCode, tErrorValue);
		}
		

	\endcode            
*/ '''             
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101Cancel(enum eOperationFlag eOperation, IEC101Object myIEC101Obj, struct sIEC101DataAttributeID *psDAID, struct sIEC101DataAttributeData *psCancelValue, struct sIEC101CommandParameters *psCancelParams, tErrorValue *ptErrorValue);
eOperation = ctypes.c_int
iec101_lib.IEC101Cancel.argtypes = [ eOperation, IEC101Object, ctypes.POINTER(sIEC101DataAttributeID), ctypes.POINTER(sIEC101DataAttributeData), ctypes.POINTER(sIEC101CommandParameters), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101Cancel.restype = ctypes.c_short

''' /*! \brief        Get IEC101 data type and data size to the returned Value.
	\ingroup      Management
		 
	\param[in]    myIEC101Obj           IEC101 object to Get Type and Size
	\param[in]    psDAID              Pointer to IEC101 Data Attribute ID
	\param[out]   psReturnedValue     Pointer to IEC101 Data Attribute Data containing only data type and data size.
	\param[out]   ptErrorValue        Pointer to a Error Value 
		 
	\return       EC_NONE on success
	\return       otherwise error code
		 
	Example Usage:
	\code

		Integer16                    i16ErrorCode        = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;
		struct sIEC101DataAttributeID      sDAID           = {0};
		struct sIEC101DataAttributeData    sReturnedValue  = {0};
		 
		// Set the Type ID for which you want to get the datatype and datasize 
		sDAID.eTypeID         = M_SP_NA_1;
		 
		//Call function to get type and size
		i16ErrorCode = IEC101GetDataTypeAndSize(myiec101Obj, &sDAID, &sReturnedValue, &tErrorValue);
		if(i16ErrorCode != EC_NONE)
		{
		   printf("Get Type IEC101 has failed: %i %i", i16ErrorCode, tErrorValue);
		}
		else
		{
		   printf("\r\n Type is : %u, Size is %u", sReturnedValue.eDataType, sReturnedValue.eDataSize);
		}
		
	\endcode 

*/ '''
 #PUBLICAPIPX Integer16 PUBLICAPISX IEC101GetDataTypeAndSize(IEC101Object myIEC101Obj, struct sIEC101DataAttributeID *psDAID, struct sIEC101DataAttributeData *psReturnedValue, tErrorValue *ptErrorValue);
iec101_lib.IEC101GetDataTypeAndSize.argtypes = [IEC101Object, ctypes.POINTER(sIEC101DataAttributeID),ctypes.POINTER(sIEC101DataAttributeData), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101GetDataTypeAndSize.restype = ctypes.c_short
	
''' /*!\brief           Send an Parameter Act command on given control Data object. 
	\ingroup        Management

	\param[in]      myIEC101Obj       IEC101 object 
	\param[in]      psDAID          Pointer to IEC101 Data Attribute ID of control that is to be Operated
	\param[in]      psOperateValue  Pointer to IEC101 Data Attribute Data (The value the control is to be set )
	\param[in]      psParaParams Pointer to IEC101 Data Attribute Parameters (Quality Paramters)
	\param[out]     ptErrorValue    Pointer to a Error Value 

	\return         EC_NONE on success
	\return         otherwise error code

	Client Example Usage:
	\code
	
		Integer16                  i16ErrorCode      = EC_NONE;
		tErrorValue                         tErrorValue       = EV_NONE;

		Float32             f32value        =   0;
		struct sIEC101DataAttributeID sDAID;
		struct sIEC101DataAttributeData sOperateValue;
		struct sIEC101ParameterActParameters sParaParams;
		
		sDAID.u16SerialPortNumber                =   1;
		sDAID.u16DataLinkAddress                =   1;
		sDAID.u16CommonAddress                  =   1;
		sDAID.eTypeID               =   P_ME_NC_1;
		sDAID.u16CommonAddress    =   1;
		sDAID.u32IOA                =   8006;

		f32value                    =   -1.2345;
		sOperateValue.eDataSize     =   FLOAT32_SIZE;
		sOperateValue.eDataType     =   FLOAT32_DATA;
		sOperateValue.pvData            =   &f32value;

		sOperateParams.eQOCQU       =   NOADDDEF;
		memset(&sOperateValue.sTimeStamp, 0, sizeof(struct sTargetTimeStamp));

		//current date 11/8/2012
		sOperateValue.sTimeStamp.u8Day              =   8;
		sOperateValue.sTimeStamp.u8Month                =   11;
		sOperateValue.sTimeStamp.u16Year                =   2012;

		//time 13.35.0
		sOperateValue.sTimeStamp.u8Hour             =   13;
		sOperateValue.sTimeStamp.u8Minute           =   36;
		sOperateValue.sTimeStamp.u8Seconds          =   0;
		sOperateValue.sTimeStamp.u16MilliSeconds        =   0;
		sOperateValue.sTimeStamp.u16MicroSeconds        =   0;
		sOperateValue.sTimeStamp.i8DSTTime          =   0; //No Day light saving time
		sOperateValue.sTimeStamp.u8DayoftheWeek     =   4;
		
		
		i16ErrorCode =  IEC101ParameterAct(myClient,&sDAID, &sOperateValue, &sOperateParams,&tErrorValue);
		if(i16ErrorCode != EC_NONE)
		{
			printf("\r\nError: IEC101ParameterAct() failed: %i %i", i16ErrorCode, tErrorValue);
		}
		
	\endcode            
*/ '''
 #PUBLICAPIPX Integer16 PUBLICAPISX IEC101ParameterAct(IEC101Object myIEC101Obj, struct sIEC101DataAttributeID *psDAID,  struct sIEC101DataAttributeData *psOperateValue, struct sIEC101ParameterActParameters *psParaParams, tErrorValue *ptErrorValue);
iec101_lib.IEC101ParameterAct.argtypes = [IEC101Object, ctypes.POINTER(sIEC101DataAttributeID), ctypes.POINTER(sIEC101DataAttributeData), ctypes.POINTER(sIEC101ParameterActParameters), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101ParameterAct.restype = ctypes.c_short
''' /*! \brief        Get IEC101 Client Status.
 *  \par          Get IEC101 Get Client connection status.
 *  \ingroup      Management
 *
 *  \param[in]    myIEC101Obj         IEC101 object 
 *  \param[in]    psDAID              Pointer to IEC101 Data Attribute ID
 *  \param[out]    peSat              Pointer to enum eStatus 
 *  \param[out]   ptErrorValue        Pointer to a Error Value
 *
 *  \return       EC_NONE on success
 *  \return       otherwise error code
 */ 
 '''
eSat = ctypes.c_int
peSat= ctypes.POINTER(eSat)
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101ClientStatus(IEC101Object myIEC101Obj, struct sIEC101DataAttributeID *psDAID, enum eStatus *peSat, tErrorValue *ptErrorValue);
iec101_lib.IEC101ClientStatus.argtypes = [IEC101Object, ctypes.POINTER(sIEC101DataAttributeID), peSat, ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101ClientStatus.restype = ctypes.c_short

''' /*! \brief        Get IEC101 Get File.
 *  \par          Get IEC101 Get file Using File Name.
 *  \ingroup      Management
 *
 *  \param[in]    myIEC101Obj         IEC101 object to Get Type and Size
 *  \param[in]    psDAID              Pointer to IEC101 Data Attribute ID
 *  \param[in]    u16FileName         File Name.
 *  \param[out]   ptErrorValue        Pointer to a Error Value
 *
 *  \return       EC_NONE on success
 *  \return       otherwise error code
 */ '''
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101GetFile(IEC101Object myIEC101Obj, struct sIEC101DataAttributeID * psDAID, Unsigned16 u16FileName, tErrorValue *ptErrorValue);

u16FileName = ctypes.c_ushort
iec101_lib.IEC101GetFile.argtypes = [IEC101Object, ctypes.POINTER(sIEC101DataAttributeID), u16FileName, ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101GetFile.restype = ctypes.c_short
	
''' /*! \brief        IEC101 List Directory
 *  \par          Get Directory List as call Backs
 *  \ingroup      Management
 *
 *  \param[in]    myIEC101Obj         IEC101 object to Get Type and Size
 *  \param[in]    psDAID              Pointer to IEC101 Data Attribute ID
 *  \param[out]   ptErrorValue        Pointer to a Error Value
 *
 *  \return       EC_NONE on success
 *  \return       otherwise error code
 */ '''
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101ListDirectory(IEC101Object myIEC101Obj, struct sIEC101DataAttributeID * psDAID, tErrorValue *ptErrorValue);
iec101_lib.IEC101ListDirectory.argtypes = [IEC101Object, ctypes.POINTER(sIEC101DataAttributeID), ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101ListDirectory.restype = ctypes.c_short
''' /*! \brief        Get IEC101 object Status.
 *  \par          Get IEC101 Get object status -  loaded, running, stoped, freed.
 *  \ingroup      Management
 *
 *  \param[in]    myIEC101Obj         IEC101 object 
 *  \param[out]   peCurrentState      Pointer to enum  eAppState   
 *  \param[out]   ptErrorValue        Pointer to a Error Value
 *
 *  \return       EC_NONE on success
 *  \return       otherwise error code
 */ '''
#PUBLICAPIPX Integer16 PUBLICAPISX GetIEC101ObjectStatus(IEC101Object myIEC101Obj, enum  eAppState  *peCurrentState, tErrorValue *ptErrorValue);
eCurrentState = ctypes.c_int
peCurrentState = ctypes.POINTER(eCurrentState)
iec101_lib.GetIEC101ObjectStatus.argtypes = [IEC101Object, peCurrentState, ctypes.POINTER(ctypes.c_short) ]
iec101_lib.GetIEC101ObjectStatus.restype = ctypes.c_short
''' /*! \brief        Get Error code String
*  \par            For particular Error code , get Error String
*  \ingroup        Management
*
*  \param[in]      psIEC101ErrorCodeDes - Pointer to struct sIEC101ErrorCode 
*
*  \return         error code string
*/ '''            
#PUBLICAPIPX void PUBLICAPISX IEC101ErrorCodeString(struct sIEC101ErrorCode *psIEC101ErrorCodeDes);
iec101_lib.IEC101ErrorCodeString.argtypes = [ctypes.POINTER(sIEC101ErrorCode)]
iec101_lib.IEC101ErrorCodeString.restype = None
''' /*! \brief        Get Error value String
*  \par            For particular Error value , get Error String
*  \ingroup        Management
*
*  \param[in]      psIEC101ErrorValueDes - Pointer to struct sIEC101ErrorValue 
*
*  \return         error value string
*/ '''
#PUBLICAPIPX void PUBLICAPISX IEC101ErrorValueString(struct sIEC101ErrorValue *psIEC101ErrorValueDes);
iec101_lib.IEC101ErrorValueString.argtypes = [ctypes.POINTER(sIEC101ErrorValue)]
iec101_lib.IEC101ErrorValueString.restype = None
''' /*! \brief             Get IEC 101 Library License information
 *  \par               Function used to get IEC 101 Library License information
 *
 *  \return            License information of library as a string of char 
 *  Example Usage:
 *  \code
 *      printf("Version number: %s", IEC101GetLibraryVersion(void));
 *  \endcode
 */ '''
#PUBLICAPIPX const Integer8 * PUBLICAPISX IEC101GetLibraryLicenseInfo(void);
iec101_lib.IEC101GetLibraryLicenseInfo.argtypes = None
iec101_lib.IEC101GetLibraryLicenseInfo.restype = ctypes.c_char_p
''' /*! \brief		  Get IEC101 Client - stop/start polling particular server in serial multi drop
 *	\par		  Get IEC101 Get Client - stop/start polling particular server in serial multi drop
 *
 *	\param[in]	  myIEC101Obj 	  IEC101 object 
 *	\param[in]	  psDAID			  Pointer to dnp3 Data Attribute ID
 *	\param[out]   bStop 			True - stop , false - start polling particular server/device
 *	\param[out]   ptErrorValue		  Pointer to a Error Value
 *
 *	\return 	  EC_NONE on success
 *	\return 	  otherwise error code
 */ '''
#PUBLICAPIPX Integer16 PUBLICAPISX IEC101ClientStopServerMultidrop(IEC101Object myIEC101Obj, struct sIEC101DataAttributeID *psDAID, Boolean bStop, tErrorValue *ptErrorValue);
bStop = ctypes.c_bool
iec101_lib.IEC101ClientStopServerMultidrop.argtypes = [IEC101Object, ctypes.POINTER(sIEC101DataAttributeID), bStop, ctypes.POINTER(ctypes.c_short) ]
iec101_lib.IEC101ClientStopServerMultidrop.restype = ctypes.c_short   

 


